#!/usr/bin/env python
#
# Copyright 2012-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import math


class test_vector_insert(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [float(x) for x in range(16)]
        expected_result = tuple(src_data)

        period = 9177
        offset = 0

        src = blocks.null_source(1)
        head = blocks.head(1, 10000000)
        ins = blocks.vector_insert_b([1], period, offset)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, head, ins, dst)
        self.tb.run()
        result_data = dst.data()

        for i in range(10000):
            if(i % period == offset):
                self.assertEqual(1, result_data[i])
            else:
                self.assertEqual(0, result_data[i])

    def test_002(self):  # insert tags and check their propagation, zero offset
        period = 11000
        offset = 0
        insert = [1.0, ] * 1000

        src = blocks.null_source(gr.sizeof_float)
        s2ts = blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, period - len(insert), "packet")
        head = blocks.head(gr.sizeof_float, 1000000)
        ins = blocks.vector_insert_f(insert, period, offset)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, s2ts, head, ins, dst)
        self.tb.run()

        expected_result = (1000, 12000, 23000, 34000, 45000, 56000, 67000)
        tags = dst.tags()
        offsets = [tag.offset for tag in tags]
        for i in range(len(expected_result)):
            self.assertEqual(expected_result[i], offsets[i])

    def test_003(self):  # insert tags and check their propagation, non-zero offset
        period = 11000
        offset = 1000
        insert = [1.0, ] * 1000

        src = blocks.null_source(gr.sizeof_float)
        s2ts = blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, period - len(insert), "packet")
        head = blocks.head(gr.sizeof_float, 1000000)
        ins = blocks.vector_insert_f(insert, period, offset)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, s2ts, head, ins, dst)
        self.tb.run()

        expected_result = (0, 11000, 22000, 33000, 44000, 55000, 66000)
        tags = dst.tags()
        offsets = [tag.offset for tag in tags]
        for i in range(len(expected_result)):
            self.assertEqual(expected_result[i], offsets[i])

    def test_004(self):  # insert tags and check their propagation, non-zero offset, multiple tags per copy region
        period = 11000
        offset = 1000
        packetlen = 2000
        insert = [1.0, ] * 1000

        src = blocks.null_source(gr.sizeof_float)
        s2ts = blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, packetlen, "packet")
        head = blocks.head(gr.sizeof_float, 1000000)
        ins = blocks.vector_insert_f(insert, period, offset)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, s2ts, head, ins, dst)
        self.tb.run()

        expected_result = (
            0,
            2000,
            4000,
            6000,
            8000,
            11000,
            13000,
            15000,
            17000,
            19000,
            22000,
            24000,
            26000)
        tags = dst.tags()
        offsets = [tag.offset for tag in tags]
        for i in range(len(expected_result)):
            self.assertEqual(expected_result[i], offsets[i])


if __name__ == '__main__':
    gr_unittest.run(test_vector_insert)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, gr_unittest, analog, blocks


def avg_mag_sqrd_c(x, alpha):
    y = [0, ]
    for xi in x:
        tmp = alpha * (xi.real * xi.real + xi.imag *
                       xi.imag) + (1 - alpha) * y[-1]
        y.append(tmp)
    return y


def avg_mag_sqrd_f(x, alpha):
    y = [0, ]
    for xi in x:
        tmp = alpha * (xi * xi) + (1 - alpha) * y[-1]
        y.append(tmp)
    return y


class test_probe_avg_mag_sqrd(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_c_001(self):
        alpha = 0.0001
        src_data = [
            1.0 + 1.0j,
            2.0 + 2.0j,
            3.0 + 3.0j,
            4.0 + 4.0j,
            5.0 + 5.0j,
            6.0 + 6.0j,
            7.0 + 7.0j,
            8.0 + 8.0j,
            9.0 + 9.0j,
            10.0 + 10.0j]
        expected_result = avg_mag_sqrd_c(src_data, alpha)[-1]

        src = blocks.vector_source_c(src_data)
        op = analog.probe_avg_mag_sqrd_c(0, alpha)

        self.tb.connect(src, op)
        self.tb.run()

        result_data = op.level()
        self.assertAlmostEqual(expected_result, result_data, 5)

    def test_cf_002(self):
        alpha = 0.0001
        src_data = [
            1.0 + 1.0j,
            2.0 + 2.0j,
            3.0 + 3.0j,
            4.0 + 4.0j,
            5.0 + 5.0j,
            6.0 + 6.0j,
            7.0 + 7.0j,
            8.0 + 8.0j,
            9.0 + 9.0j,
            10.0 + 10.0j]
        expected_result = avg_mag_sqrd_c(src_data, alpha)[0:-1]

        src = blocks.vector_source_c(src_data)
        op = analog.probe_avg_mag_sqrd_cf(0, alpha)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)

    def test_f_003(self):
        alpha = 0.0001
        src_data = [1.0, 2.0, 3.0, 4.0, 5.0,
                    6.0, 7.0, 8.0, 9.0, 10.0]
        expected_result = avg_mag_sqrd_f(src_data, alpha)[-1]

        src = blocks.vector_source_f(src_data)
        op = analog.probe_avg_mag_sqrd_f(0, alpha)

        self.tb.connect(src, op)
        self.tb.run()

        result_data = op.level()
        self.assertAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_probe_avg_mag_sqrd)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_add_mult_v(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def help_ss(self, size, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_s(s[1])
            srcv = blocks.stream_to_vector(gr.sizeof_short, size)
            self.tb.connect(src, srcv)
            self.tb.connect(srcv, (op, s[0]))

        rhs = blocks.vector_to_stream(gr.sizeof_short, size)
        dst = blocks.vector_sink_s()
        self.tb.connect(op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ii(self, size, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_i(s[1])
            srcv = blocks.stream_to_vector(gr.sizeof_int, size)
            self.tb.connect(src, srcv)
            self.tb.connect(srcv, (op, s[0]))

        rhs = blocks.vector_to_stream(gr.sizeof_int, size)
        dst = blocks.vector_sink_i()
        self.tb.connect(op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ff(self, size, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_f(s[1])
            srcv = blocks.stream_to_vector(gr.sizeof_float, size)
            self.tb.connect(src, srcv)
            self.tb.connect(srcv, (op, s[0]))

        rhs = blocks.vector_to_stream(gr.sizeof_float, size)
        dst = blocks.vector_sink_f()
        self.tb.connect(op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_cc(self, size, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_c(s[1])
            srcv = blocks.stream_to_vector(gr.sizeof_gr_complex, size)
            self.tb.connect(src, srcv)
            self.tb.connect(srcv, (op, s[0]))

        rhs = blocks.vector_to_stream(gr.sizeof_gr_complex, size)
        dst = blocks.vector_sink_c()
        self.tb.connect(op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_const_ss(self, src_data, exp_data, op):
        src = blocks.vector_source_s(src_data)
        srcv = blocks.stream_to_vector(gr.sizeof_short, len(src_data))
        rhs = blocks.vector_to_stream(gr.sizeof_short, len(src_data))
        dst = blocks.vector_sink_s()
        self.tb.connect(src, srcv, op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_const_ii(self, src_data, exp_data, op):
        src = blocks.vector_source_i(src_data)
        srcv = blocks.stream_to_vector(gr.sizeof_int, len(src_data))
        rhs = blocks.vector_to_stream(gr.sizeof_int, len(src_data))
        dst = blocks.vector_sink_i()
        self.tb.connect(src, srcv, op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_const_ff(self, src_data, exp_data, op):
        src = blocks.vector_source_f(src_data)
        srcv = blocks.stream_to_vector(gr.sizeof_float, len(src_data))
        rhs = blocks.vector_to_stream(gr.sizeof_float, len(src_data))
        dst = blocks.vector_sink_f()
        self.tb.connect(src, srcv, op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_const_cc(self, src_data, exp_data, op):
        src = blocks.vector_source_c(src_data)
        srcv = blocks.stream_to_vector(gr.sizeof_gr_complex, len(src_data))
        rhs = blocks.vector_to_stream(gr.sizeof_gr_complex, len(src_data))
        dst = blocks.vector_sink_c()
        self.tb.connect(src, srcv, op, rhs, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    # add_vXX

    def test_add_vss_one(self):
        src1_data = [1, ]
        src2_data = [2, ]
        src3_data = [3, ]
        expected_result = [6, ]
        op = blocks.add_ss(1)
        self.help_ss(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vss_five(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [6, 7, 8, 9, 10]
        src3_data = [11, 12, 13, 14, 15]
        expected_result = [18, 21, 24, 27, 30]
        op = blocks.add_ss(5)
        self.help_ss(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vii_one(self):
        src1_data = [1, ]
        src2_data = [2, ]
        src3_data = [3, ]
        expected_result = [6, ]
        op = blocks.add_ii(1)
        self.help_ii(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vii_five(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [6, 7, 8, 9, 10]
        src3_data = [11, 12, 13, 14, 15]
        expected_result = [18, 21, 24, 27, 30]
        op = blocks.add_ii(5)
        self.help_ii(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vff_one(self):
        src1_data = [1.0, ]
        src2_data = [2.0, ]
        src3_data = [3.0, ]
        expected_result = [6.0, ]
        op = blocks.add_ff(1)
        self.help_ff(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vff_five(self):
        src1_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src2_data = [6.0, 7.0, 8.0, 9.0, 10.0]
        src3_data = [11.0, 12.0, 13.0, 14.0, 15.0]
        expected_result = [18.0, 21.0, 24.0, 27.0, 30.0]
        op = blocks.add_ff(5)
        self.help_ff(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vcc_one(self):
        src1_data = [1.0 + 2.0j, ]
        src2_data = [3.0 + 4.0j, ]
        src3_data = [5.0 + 6.0j, ]
        expected_result = [9.0 + 12j, ]
        op = blocks.add_cc(1)
        self.help_cc(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_add_vcc_five(self):
        src1_data = [
            1.0 + 2.0j,
            3.0 + 4.0j,
            5.0 + 6.0j,
            7.0 + 8.0j,
            9.0 + 10.0j]
        src2_data = [
            11.0 + 12.0j,
            13.0 + 14.0j,
            15.0 + 16.0j,
            17.0 + 18.0j,
            19.0 + 20.0j]
        src3_data = [
            21.0 + 22.0j,
            23.0 + 24.0j,
            25.0 + 26.0j,
            27.0 + 28.0j,
            29.0 + 30.0j]
        expected_result = [
            33.0 + 36.0j,
            39.0 + 42.0j,
            45.0 + 48.0j,
            51.0 + 54.0j,
            57.0 + 60.0j]
        op = blocks.add_cc(5)
        self.help_cc(5, (src1_data, src2_data, src3_data), expected_result, op)

    # add_const_vXX

    def test_add_const_vss_one(self):
        src_data = [1, ]
        op = blocks.add_const_vss((2,))
        exp_data = [3, ]
        self.help_const_ss(src_data, exp_data, op)

    def test_add_const_vss_five(self):
        src_data = [1, 2, 3, 4, 5]
        op = blocks.add_const_vss((6, 7, 8, 9, 10))
        exp_data = [7, 9, 11, 13, 15]
        self.help_const_ss(src_data, exp_data, op)

    def test_add_const_vii_one(self):
        src_data = [1, ]
        op = blocks.add_const_vii((2,))
        exp_data = [3, ]
        self.help_const_ii(src_data, exp_data, op)

    def test_add_const_vii_five(self):
        src_data = [1, 2, 3, 4, 5]
        op = blocks.add_const_vii((6, 7, 8, 9, 10))
        exp_data = [7, 9, 11, 13, 15]
        self.help_const_ii(src_data, exp_data, op)

    def test_add_const_vff_one(self):
        src_data = [1.0, ]
        op = blocks.add_const_vff((2.0,))
        exp_data = [3.0, ]
        self.help_const_ff(src_data, exp_data, op)

    def test_add_const_vff_five(self):
        src_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        op = blocks.add_const_vff((6.0, 7.0, 8.0, 9.0, 10.0))
        exp_data = [7.0, 9.0, 11.0, 13.0, 15.0]
        self.help_const_ff(src_data, exp_data, op)

    def test_add_const_vcc_one(self):
        src_data = [1.0 + 2.0j, ]
        op = blocks.add_const_vcc((2.0 + 3.0j,))
        exp_data = [3.0 + 5.0j, ]
        self.help_const_cc(src_data, exp_data, op)

    def test_add_const_vcc_five(self):
        src_data = [
            1.0 + 2.0j,
            3.0 + 4.0j,
            5.0 + 6.0j,
            7.0 + 8.0j,
            9.0 + 10.0j]
        op = blocks.add_const_vcc(
            (11.0 + 12.0j,
             13.0 + 14.0j,
             15.0 + 16.0j,
             17.0 + 18.0j,
             19.0 + 20.0j))
        exp_data = [
            12.0 + 14.0j,
            16.0 + 18.0j,
            20.0 + 22.0j,
            24.0 + 26.0j,
            28.0 + 30.0j]
        self.help_const_cc(src_data, exp_data, op)

    # multiply_vXX

    def test_multiply_vss_one(self):
        src1_data = [1, ]
        src2_data = [2, ]
        src3_data = [3, ]
        expected_result = [6, ]
        op = blocks.multiply_ss(1)
        self.help_ss(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vss_five(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [6, 7, 8, 9, 10]
        src3_data = [11, 12, 13, 14, 15]
        expected_result = [66, 168, 312, 504, 750]
        op = blocks.multiply_ss(5)
        self.help_ss(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vii_one(self):
        src1_data = [1, ]
        src2_data = [2, ]
        src3_data = [3, ]
        expected_result = [6, ]
        op = blocks.multiply_ii(1)
        self.help_ii(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vii_five(self):
        src1_data = [1, 2, 3, 4, 5]
        src2_data = [6, 7, 8, 9, 10]
        src3_data = [11, 12, 13, 14, 15]
        expected_result = [66, 168, 312, 504, 750]
        op = blocks.multiply_ii(5)
        self.help_ii(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vff_one(self):
        src1_data = [1.0, ]
        src2_data = [2.0, ]
        src3_data = [3.0, ]
        expected_result = [6.0, ]
        op = blocks.multiply_ff(1)
        self.help_ff(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vff_five(self):
        src1_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        src2_data = [6.0, 7.0, 8.0, 9.0, 10.0]
        src3_data = [11.0, 12.0, 13.0, 14.0, 15.0]
        expected_result = [66.0, 168.0, 312.0, 504.0, 750.0]
        op = blocks.multiply_ff(5)
        self.help_ff(5, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vcc_one(self):
        src1_data = [1.0 + 2.0j, ]
        src2_data = [3.0 + 4.0j, ]
        src3_data = [5.0 + 6.0j, ]
        expected_result = [-85 + 20j, ]
        op = blocks.multiply_cc(1)
        self.help_cc(1, (src1_data, src2_data, src3_data), expected_result, op)

    def test_multiply_vcc_five(self):
        src1_data = [
            1.0 + 2.0j,
            3.0 + 4.0j,
            5.0 + 6.0j,
            7.0 + 8.0j,
            9.0 + 10.0j]
        src2_data = [
            11.0 + 12.0j,
            13.0 + 14.0j,
            15.0 + 16.0j,
            17.0 + 18.0j,
            19.0 + 20.0j]
        src3_data = [
            21.0 + 22.0j,
            23.0 + 24.0j,
            25.0 + 26.0j,
            27.0 + 28.0j,
            29.0 + 30.0j]
        expected_result = [-1021.0 + 428.0j, -2647.0 + 1754.0j, -
                           4945.0 + 3704.0j, -8011.0 + 6374.0j, -11941.0 + 9860.0j]
        op = blocks.multiply_cc(5)
        self.help_cc(5, (src1_data, src2_data, src3_data), expected_result, op)

    # multiply_const_vXX

    def test_multiply_const_vss_one(self):
        src_data = [2, ]
        op = blocks.multiply_const_vss((3,))
        exp_data = [6, ]
        self.help_const_ss(src_data, exp_data, op)

    def test_multiply_const_vss_five(self):
        src_data = [1, 2, 3, 4, 5]
        op = blocks.multiply_const_vss([6, 7, 8, 9, 10])
        exp_data = [6, 14, 24, 36, 50]
        self.help_const_ss(src_data, exp_data, op)

    def test_multiply_const_vii_one(self):
        src_data = [2, ]
        op = blocks.multiply_const_vii((3,))
        exp_data = [6, ]
        self.help_const_ii(src_data, exp_data, op)

    def test_multiply_const_vii_five(self):
        src_data = [1, 2, 3, 4, 5]
        op = blocks.multiply_const_vii((6, 7, 8, 9, 10))
        exp_data = [6, 14, 24, 36, 50]
        self.help_const_ii(src_data, exp_data, op)

    def test_multiply_const_vff_one(self):
        src_data = [2.0, ]
        op = blocks.multiply_const_vff((3.0,))
        exp_data = [6.0, ]
        self.help_const_ff(src_data, exp_data, op)

    def test_multiply_const_vff_five(self):
        src_data = [1.0, 2.0, 3.0, 4.0, 5.0]
        op = blocks.multiply_const_vff((6.0, 7.0, 8.0, 9.0, 10.0))
        exp_data = [6.0, 14.0, 24.0, 36.0, 50.0]
        self.help_const_ff(src_data, exp_data, op)

    def test_multiply_const_vcc_one(self):
        src_data = [1.0 + 2.0j, ]
        op = blocks.multiply_const_vcc((2.0 + 3.0j,))
        exp_data = [-4.0 + 7.0j, ]
        self.help_const_cc(src_data, exp_data, op)

    def test_multiply_const_vcc_five(self):
        src_data = [
            1.0 + 2.0j,
            3.0 + 4.0j,
            5.0 + 6.0j,
            7.0 + 8.0j,
            9.0 + 10.0j]
        op = blocks.multiply_const_vcc(
            (11.0 + 12.0j, 13.0 + 14.0j, 15.0 + 16.0j, 17.0 + 18.0j, 19.0 + 20.0j))
        exp_data = [-13.0 + 34.0j, -17.0 + 94.0j, -
                    21.0 + 170.0j, -25.0 + 262.0j, -29.0 + 370.0j]
        self.help_const_cc(src_data, exp_data, op)


if __name__ == '__main__':
    gr_unittest.run(test_add_mult_v)



# Copyright 2016 Free Software Foundation, Inc.
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-2.0-or-later
#
"""
Converter for legacy block definitions in XML format

- Cheetah expressions that can not be converted are passed to Cheetah for now
- Instead of generating a Block subclass directly a string representation is
  used and evaluated. This is slower / lamer but allows us to show the user
  how a converted definition would look like
"""


from collections import OrderedDict, defaultdict
from itertools import chain

from ..core.io import yaml
from . import cheetah_converter, xml

current_file_format = 1
reserved_block_keys = ('import', )  # todo: add more keys


def from_xml(filename):
    """Load block description from xml file"""
    element, version_info = xml.load(filename, 'block.dtd')

    try:
        data = convert_block_xml(element)
    except NameError:
        raise ValueError('Conversion failed', filename)

    return data


def dump(data, stream):
    out = yaml.dump(data)

    replace = [
        ('parameters:', '\nparameters:'),
        ('inputs:', '\ninputs:'),
        ('outputs:', '\noutputs:'),
        ('templates:', '\ntemplates:'),
        ('documentation:', '\ndocumentation:'),
        ('file_format:', '\nfile_format:'),
    ]
    for r in replace:
        out = out.replace(*r)
    prefix = '# auto-generated by grc.converter\n\n'
    stream.write(prefix + out)


no_value = object()
dummy = cheetah_converter.DummyConverter()


def convert_block_xml(node):
    converter = cheetah_converter.Converter(names={
        param_node.findtext('key'): {
            opt_node.text.split(':')[0]
            for opt_node in next(param_node.iterfind('option'), param_node).iterfind('opt')
        } for param_node in node.iterfind('param')
    })

    block_id = node.findtext('key')
    if block_id in reserved_block_keys:
        block_id += '_'

    data = OrderedDict()
    data['id'] = block_id
    data['label'] = node.findtext('name') or no_value
    data['category'] = node.findtext('category') or no_value
    data['flags'] = [n.text for n in node.findall('flags')]
    data['flags'] += ['show_id'] if block_id.startswith('variable') else []
    if not data['flags']:
        data['flags'] = no_value

    data['parameters'] = [convert_param_xml(param_node, converter.to_python_dec)
                          for param_node in node.iterfind('param')] or no_value
    # data['params'] = {p.pop('key'): p for p in data['params']}

    data['inputs'] = [convert_port_xml(port_node, converter.to_python_dec)
                      for port_node in node.iterfind('sink')] or no_value

    data['outputs'] = [convert_port_xml(port_node, converter.to_python_dec)
                       for port_node in node.iterfind('source')] or no_value
    data['value'] = (
        converter.to_python_dec(node.findtext('var_value')) or
        ('${ value }' if block_id.startswith('variable') else no_value)
    )

    data['asserts'] = [converter.to_python_dec(check_node.text)
                       for check_node in node.iterfind('check')] or no_value

    data['templates'] = convert_templates(
        node, converter.to_mako, block_id) or no_value

    docs = node.findtext('doc')
    if docs:
        docs = docs.strip().replace('\\\n', '')
        data['documentation'] = yaml.MultiLineString(docs)

    data['file_format'] = current_file_format

    data = OrderedDict((key, value)
                       for key, value in data.items() if value is not no_value)
    auto_hide_params_for_item_sizes(data)

    return data


def auto_hide_params_for_item_sizes(data):
    item_size_templates = []
    vlen_templates = []
    for port in chain(*[data.get(direction, []) for direction in ['inputs', 'outputs']]):
        for key in ['dtype', 'multiplicity']:
            item_size_templates.append(str(port.get(key, '')))
        vlen_templates.append(str(port.get('vlen', '')))
    item_size_templates = ' '.join(
        value for value in item_size_templates if '${' in value)
    vlen_templates = ' '.join(
        value for value in vlen_templates if '${' in value)

    for param in data.get('parameters', []):
        if param['id'] in item_size_templates:
            param.setdefault('hide', 'part')
        if param['id'] in vlen_templates:
            param.setdefault('hide', "${ 'part' if vlen == 1 else 'none' }")


def convert_templates(node, convert, block_id=''):
    templates = OrderedDict()

    imports = '\n'.join(convert(import_node.text)
                        for import_node in node.iterfind('import'))
    if '\n' in imports:
        imports = yaml.MultiLineString(imports)
    templates['imports'] = imports or no_value

    templates['var_make'] = convert(
        node.findtext('var_make') or '') or no_value

    make = convert(node.findtext('make') or '')
    if make:
        check_mako_template(block_id, make)
    if '\n' in make:
        make = yaml.MultiLineString(make)
    templates['make'] = make or no_value

    templates['callbacks'] = [
        convert(cb_node.text) for cb_node in node.iterfind('callback')
    ] or no_value

    return OrderedDict((key, value) for key, value in templates.items() if value is not no_value)


def convert_param_xml(node, convert):
    param = OrderedDict()
    param['id'] = node.findtext('key').strip()
    param['label'] = node.findtext('name').strip()
    param['category'] = node.findtext('tab') or no_value

    param['dtype'] = convert(node.findtext('type') or '')
    param['default'] = node.findtext('value') or no_value

    options = yaml.ListFlowing(on.findtext('key')
                               for on in node.iterfind('option'))
    option_labels = yaml.ListFlowing(on.findtext(
        'name') for on in node.iterfind('option'))
    param['options'] = options or no_value
    if not all(str(o).title() == l for o, l in zip(options, option_labels)):
        param['option_labels'] = option_labels

    attributes = defaultdict(yaml.ListFlowing)
    for option_n in node.iterfind('option'):
        for opt_n in option_n.iterfind('opt'):
            key, value = opt_n.text.split(':', 2)
            attributes[key].append(value)
    param['option_attributes'] = dict(attributes) or no_value

    param['hide'] = convert(node.findtext('hide')) or no_value

    return OrderedDict((key, value) for key, value in param.items() if value is not no_value)


def convert_port_xml(node, convert):
    port = OrderedDict()
    label = node.findtext('name')
    # default values:
    port['label'] = label if label not in ('in', 'out') else no_value

    dtype = convert(node.findtext('type'))
    # TODO: detect dyn message ports
    port['domain'] = domain = 'message' if dtype == 'message' else 'stream'
    if domain == 'message':
        port['id'], port['label'] = label, no_value
    else:
        port['dtype'] = dtype
        vlen = node.findtext('vlen')
        port['vlen'] = int(vlen) if vlen and vlen.isdigit(
        ) else convert(vlen) or no_value

    port['multiplicity'] = convert(node.findtext('nports')) or no_value
    port['optional'] = bool(node.findtext('optional')) or no_value
    port['hide'] = convert(node.findtext('hide')) or no_value

    return OrderedDict((key, value) for key, value in port.items() if value is not no_value)


def check_mako_template(block_id, expr):
    import sys
    from mako.template import Template
    try:
        Template(expr)
    except Exception as error:
        print(block_id, expr, type(error), error,
              '', sep='\n', file=sys.stderr)



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import fec

from _qa_helper import _qa_helper
from _qa_helper_async import _qa_helper_async


class test_fecapi_repetition(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_parallelism0_00(self):
        frame_size = 30
        rep = 3
        enc = fec.repetition_encoder_make(frame_size * 8, rep)
        dec = fec.repetition_decoder.make(frame_size * 8, rep)
        threading = None
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism0_01(self):
        frame_size = 30
        rep = 3
        enc = fec.repetition_encoder_make(frame_size * 8, rep)
        dec = fec.repetition_decoder.make(frame_size * 8, rep)
        threading = 'ordinary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism0_02(self):
        frame_size = 30
        rep = 3
        enc = fec.repetition_encoder_make(frame_size * 8, rep)
        dec = fec.repetition_decoder.make(frame_size * 8, rep)
        threading = 'capillary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_00(self):
        frame_size = 30
        rep = 3
        enc = list(map((lambda a: fec.repetition_encoder_make(
            frame_size * 8, rep)), list(range(0, 1))))
        dec = list(map((lambda a: fec.repetition_decoder.make(
            frame_size * 8, rep)), list(range(0, 1))))
        threading = None
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_01(self):
        frame_size = 30
        rep = 3
        enc = list(map((lambda a: fec.repetition_encoder_make(
            frame_size * 8, rep)), list(range(0, 1))))
        dec = list(map((lambda a: fec.repetition_decoder.make(
            frame_size * 8, rep)), list(range(0, 1))))
        threading = 'ordinary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_02(self):
        frame_size = 300
        rep = 3
        enc = list(map((lambda a: fec.repetition_encoder_make(
            frame_size * 8, rep)), list(range(0, 1))))
        dec = list(map((lambda a: fec.repetition_decoder.make(
            frame_size * 8, rep)), list(range(0, 1))))
        threading = 'capillary'
        self.test = _qa_helper(10 * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_03(self):
        frame_size = 30
        rep = 3
        dims = 10
        enc = list(map((lambda a: fec.repetition_encoder_make(
            frame_size * 8, rep)), list(range(0, dims))))
        dec = list(map((lambda a: fec.repetition_decoder.make(
            frame_size * 8, rep)), list(range(0, dims))))
        threading = 'ordinary'
        self.test = _qa_helper(dims * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_parallelism1_04(self):
        frame_size = 30
        rep = 3
        dims = 16
        enc = list(map((lambda a: fec.repetition_encoder_make(
            frame_size * 8, rep)), list(range(0, dims))))
        dec = list(map((lambda a: fec.repetition_decoder.make(
            frame_size * 8, rep)), list(range(0, dims))))
        threading = 'capillary'
        self.test = _qa_helper(dims * frame_size, enc, dec, threading)
        self.tb.connect(self.test)
        self.tb.run()

        data_in = self.test.snk_input.data()
        data_out = self.test.snk_output.data()

        self.assertSequenceEqualGR(data_in, data_out)

    def test_async_00(self):
        frame_size = 30
        rep = 3
        enc = fec.repetition_encoder_make(frame_size * 8, rep)
        dec = fec.repetition_decoder.make(frame_size * 8, rep)
        for packed in [True, False]:
            for rev_pack in [True, False]:
                with self.subTest(packed=packed, rev_pack=rev_pack):
                    self.test = _qa_helper_async(
                        frame_size, enc, dec, packed, rev_pack)
                    self.test.run()

                    data_in = self.test.snk_input.data()
                    data_out = self.test.snk_output.data()

                    self.assertSequenceEqualGR(data_in, data_out)


if __name__ == '__main__':
    gr_unittest.run(test_fecapi_repetition)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, analog, blocks


class test_pwr_squelch(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_pwr_squelch_001(self):
        # Test set/gets

        alpha = 0.0001

        thr1 = 10
        thr2 = 20
        ramp = 1
        ramp2 = 2
        gate = True
        gate2 = False

        op = analog.pwr_squelch_cc(thr1, alpha, ramp, gate)

        op.set_threshold(thr2)
        t = op.threshold()
        self.assertEqual(thr2, t)

        op.set_ramp(ramp2)
        r = op.ramp()
        self.assertEqual(ramp2, r)

        op.set_gate(gate2)
        g = op.gate()
        self.assertEqual(gate2, g)

    def test_pwr_squelch_002(self):
        # Test runtime, gate=True
        alpha = 0.0001
        thr = -25

        src_data = [float(x) / 10.0 for x in range(1, 40)]
        src = blocks.vector_source_c(src_data)
        op = analog.pwr_squelch_cc(thr, alpha)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        expected_result = src_data
        expected_result[0:20] = 20 * [0, ]

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 4)

    def test_pwr_squelch_003(self):
        # Test set/gets

        alpha = 0.0001

        thr1 = 10
        thr2 = 20
        ramp = 1
        ramp2 = 2
        gate = True
        gate2 = False

        op = analog.pwr_squelch_ff(thr1, alpha, ramp, gate)

        op.set_threshold(thr2)
        t = op.threshold()
        self.assertEqual(thr2, t)

        op.set_ramp(ramp2)
        r = op.ramp()
        self.assertEqual(ramp2, r)

        op.set_gate(gate2)
        g = op.gate()
        self.assertEqual(gate2, g)

    def test_pwr_squelch_004(self):
        alpha = 0.0001
        thr = -25

        src_data = [float(x) / 10.0 for x in range(1, 40)]
        src = blocks.vector_source_f(src_data)
        op = analog.pwr_squelch_ff(thr, alpha)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        expected_result = src_data
        expected_result[0:20] = 20 * [0, ]

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_pwr_squelch)



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math


class test_min(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [0, 0.2, -0.25, 0, 12, 0]
        expected_result = [float(min(src_data)), ]

        src = blocks.vector_source_f(src_data)
        s2v = blocks.stream_to_vector(gr.sizeof_float, len(src_data))
        op = blocks.min_ff(len(src_data))
        dst = blocks.vector_sink_f()

        self.tb.connect(src, s2v, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_002(self):
        src_data = [-100, -99, -98, -97, -96, -1]
        expected_result = [float(min(src_data)), ]

        src = blocks.vector_source_f(src_data)
        s2v = blocks.stream_to_vector(gr.sizeof_float, len(src_data))
        op = blocks.min_ff(len(src_data))
        dst = blocks.vector_sink_f()

        self.tb.connect(src, s2v, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_003(self):
        src_data0 = [0, 2, -3, 0, 12, 0]
        src_data1 = [1, 1, 1, 1, 1, 1]

        expected_result = [float(min(x, y))
                           for x, y in zip(src_data0, src_data1)]

        src0 = blocks.vector_source_f(src_data0)
        src1 = blocks.vector_source_f(src_data1)
        op = blocks.min_ff(1)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_004(self):
        dim = 2
        src_data0 = [0, 2, -3, 0, 12, 0]
        src_data1 = [1, 1, 1, 1, 1, 1]

        expected_data = []
        tmp = [float(min(x, y)) for x, y in zip(src_data0, src_data1)]
        for i in range(len(tmp) / dim):
            expected_data.append(float(min(tmp[i * dim:(i + 1) * dim])))

        src0 = blocks.vector_source_f(src_data0)
        s2v0 = blocks.stream_to_vector(gr.sizeof_float, dim)
        src1 = blocks.vector_source_f(src_data1)
        s2v1 = blocks.stream_to_vector(gr.sizeof_float, dim)
        op = blocks.min_ff(dim)
        dst = blocks.vector_sink_f()

        self.tb.connect(src0, s2v0, (op, 0))
        self.tb.connect(src1, s2v1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_s001(self):
        src_data = [0, 2, -3, 0, 12, 0]
        expected_result = [min(src_data), ]

        src = blocks.vector_source_s(src_data)
        s2v = blocks.stream_to_vector(gr.sizeof_short, len(src_data))
        op = blocks.min_ss(len(src_data))
        dst = blocks.vector_sink_s()

        self.tb.connect(src, s2v, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_s002(self):
        src_data = [-100, -99, -98, -97, -96, -1]
        expected_result = [min(src_data), ]

        src = blocks.vector_source_s(src_data)
        s2v = blocks.stream_to_vector(gr.sizeof_short, len(src_data))
        op = blocks.min_ss(len(src_data))
        dst = blocks.vector_sink_s()

        self.tb.connect(src, s2v, op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_s003(self):
        src_data0 = [0, 2, -3, 0, 12, 0]
        src_data1 = [1, 1, 1, 1, 1, 1]

        expected_result = [min(x, y) for x, y in zip(src_data0, src_data1)]

        src0 = blocks.vector_source_s(src_data0)
        src1 = blocks.vector_source_s(src_data1)
        op = blocks.min_ss(1)
        dst = blocks.vector_sink_s()

        self.tb.connect(src0, (op, 0))
        self.tb.connect(src1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def stest_s004(self):
        dim = 2
        src_data0 = [0, 2, -3, 0, 12, 0]
        src_data1 = [1, 1, 1, 1, 1, 1]

        expected_data = []
        tmp = [min(x, y) for x, y in zip(src_data0, src_data1)]
        for i in range(len(tmp) / dim):
            expected_data.append(min(tmp[i * dim:(i + 1) * dim]))

        src0 = blocks.vector_source_s(src_data0)
        s2v0 = blocks.stream_to_vector(gr.sizeof_short, dim)
        src1 = blocks.vector_source_s(src_data1)
        s2v1 = blocks.stream_to_vector(gr.sizeof_short, dim)
        op = blocks.min_ss(dim)
        dst = blocks.vector_sink_s()

        self.tb.connect(src0, s2v0, (op, 0))
        self.tb.connect(src1, s2v1, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_min)



#!/usr/bin/env python
#
# Copyright 2013,2017 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks

import math
import random


def make_random_complex_tuple(L, scale=1):
    result = []
    for x in range(L):
        result.append(scale * complex(2 * random.random() - 1,
                                      2 * random.random() - 1))
    return tuple(result)


def make_random_float_tuple(L, scale=1):
    result = []
    for x in range(L):
        result.append(scale * (2 * random.random() - 1))
    return tuple(result)


class test_moving_average(gr_unittest.TestCase):
    def assertListAlmostEqual(self, list1, list2, tol):
        self.assertEqual(len(list1), len(list2))
        for a, b in zip(list1, list2):
            self.assertAlmostEqual(a, b, tol)

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    # These tests will always pass and are therefore useless. 100 random numbers [-1,1) are
    # getting summed up and scaled with 0.001. Then, an assertion verifies a result near 0,
    # which is the case even if the block is malfunctioning.

    def test_01(self):
        tb = self.tb

        N = 10000
        data = make_random_float_tuple(N, 1)
        expected_result = N * [0, ]

        src = blocks.vector_source_f(data, False)
        op = blocks.moving_average_ff(100, 0.001)
        dst = blocks.vector_sink_f()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        # make sure result is close to zero
        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 1)

    def test_02(self):
        tb = self.tb

        N = 10000
        data = make_random_complex_tuple(N, 1)
        expected_result = N * [0, ]

        src = blocks.vector_source_c(data, False)
        op = blocks.moving_average_cc(100, 0.001)
        dst = blocks.vector_sink_c()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        # make sure result is close to zero
        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 1)

    # This tests implement own moving average to verify correct behaviour of
    # the block

    def test_vector_int(self):
        tb = self.tb

        vlen = 5
        N = 10 * vlen
        data = make_random_float_tuple(N, 2**10)
        data = [int(d * 1000) for d in data]
        src = blocks.vector_source_i(data, False)
        one_to_many = blocks.stream_to_streams(gr.sizeof_int, vlen)
        one_to_vector = blocks.stream_to_vector(gr.sizeof_int, vlen)
        many_to_vector = blocks.streams_to_vector(gr.sizeof_int, vlen)
        isolated = [blocks.moving_average_ii(100, 1) for i in range(vlen)]
        dut = blocks.moving_average_ii(100, 1, vlen=vlen)
        dut_dst = blocks.vector_sink_i(vlen=vlen)
        ref_dst = blocks.vector_sink_i(vlen=vlen)

        tb.connect(src, one_to_many)
        tb.connect(src, one_to_vector, dut, dut_dst)
        tb.connect(many_to_vector, ref_dst)
        for idx, single in enumerate(isolated):
            tb.connect((one_to_many, idx), single, (many_to_vector, idx))

        tb.run()

        dut_data = dut_dst.data()
        ref_data = ref_dst.data()

        # make sure result is close to zero
        self.assertEqual(dut_data, ref_data)

    def test_vector_complex(self):
        tb = self.tb

        vlen = 5
        N = 10 * vlen
        data = make_random_complex_tuple(N, 2**10)
        src = blocks.vector_source_c(data, False)
        one_to_many = blocks.stream_to_streams(gr.sizeof_gr_complex, vlen)
        one_to_vector = blocks.stream_to_vector(gr.sizeof_gr_complex, vlen)
        many_to_vector = blocks.streams_to_vector(gr.sizeof_gr_complex, vlen)
        isolated = [blocks.moving_average_cc(100, 1) for i in range(vlen)]
        dut = blocks.moving_average_cc(100, 1, vlen=vlen)
        dut_dst = blocks.vector_sink_c(vlen=vlen)
        ref_dst = blocks.vector_sink_c(vlen=vlen)

        tb.connect(src, one_to_many)
        tb.connect(src, one_to_vector, dut, dut_dst)
        tb.connect(many_to_vector, ref_dst)
        for idx, single in enumerate(isolated):
            tb.connect((one_to_many, idx), single, (many_to_vector, idx))

        tb.run()

        dut_data = dut_dst.data()
        ref_data = ref_dst.data()

        # make sure result is close to zero
        self.assertListAlmostEqual(dut_data, ref_data, tol=3)

    def test_complex_scalar(self):
        tb = self.tb

        N = 10000  # number of samples
        history = 100  # num of samples to average
        data = make_random_complex_tuple(N, 1)  # generate random data

        #  pythonic MA filter
        data_padded = (history - 1) * \
            [complex(0.0, 0.0)] + list(data)  # history
        expected_result = []
        moving_sum = sum(data_padded[:history - 1])
        for i in range(N):
            moving_sum += data_padded[i + history - 1]
            expected_result.append(moving_sum)
            moving_sum -= data_padded[i]

        src = blocks.vector_source_c(data, False)
        op = blocks.moving_average_cc(history, 1)
        dst = blocks.vector_sink_c()

        tb.connect(src, op)
        tb.connect(op, dst)
        tb.run()

        dst_data = dst.data()

        # make sure result is close to zero
        self.assertListAlmostEqual(expected_result, dst_data, 4)


if __name__ == '__main__':
    gr_unittest.run(test_moving_average)



#!/usr/bin/env python
#
# Copyright 2012-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks
import math


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


def run_test(tb, channel, fft_rotate, fft_filter):
    N = 1000         # number of samples to use
    M = 5            # Number of channels
    fs = 5000.0      # baseband sampling rate
    ifs = M * fs       # input samp rate to decimator

    taps = filter.firdes.low_pass_2(1, ifs, fs / 2, fs / 10,
                                    attenuation_dB=80,
                                    window=fft.window.WIN_BLACKMAN_hARRIS)

    signals = list()
    add = blocks.add_cc()
    freqs = [-230., 121., 110., -513., 203.]
    Mch = ((len(freqs) - 1) // 2 + channel) % len(freqs)
    for i in range(len(freqs)):
        f = freqs[i] + (M // 2 - M + i + 1) * fs
        data = sig_source_c(ifs, f, 1, N)
        signals.append(blocks.vector_source_c(data))
        tb.connect(signals[i], (add, i))

    s2ss = blocks.stream_to_streams(gr.sizeof_gr_complex, M)
    pfb = filter.pfb_decimator_ccf(M, taps, channel, fft_rotate, fft_filter)
    snk = blocks.vector_sink_c()

    tb.connect(add, s2ss)
    for i in range(M):
        tb.connect((s2ss, i), (pfb, i))
    tb.connect(pfb, snk)
    tb.run()

    L = len(snk.data())

    # Adjusted phase rotations for data
    phase = [0.11058476216852586,
             4.5108246571401693,
             3.9739891674564594,
             2.2820531095511924,
             1.3782797467397869]
    phase = phase[channel]

    # Filter delay is the normal delay of each arm
    tpf = math.ceil(len(taps) / float(M))
    delay = -(tpf - 1.0) / 2.0
    delay = int(delay)

    # Create a time scale that's delayed to match the filter delay
    t = [float(x) / fs for x in range(delay, L + delay)]

    # Create known data as complex sinusoids for the baseband freq
    # of the extracted channel is due to decimator output order.
    expected_data = [
        math.cos(
            2. *
            math.pi *
            freqs[Mch] *
            x +
            phase) +
        1j *
        math.sin(
            2. *
            math.pi *
            freqs[Mch] *
            x +
            phase) for x in t]
    dst_data = snk.data()

    return (dst_data, expected_data)


class test_pfb_decimator(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        Ntest = 50
        dst_data0, expected_data0 = run_test(self.tb, 0, False, False)
        dst_data1, expected_data1 = run_test(self.tb, 0, False, True)
        dst_data2, expected_data2 = run_test(self.tb, 0, True, False)
        dst_data3, expected_data3 = run_test(self.tb, 0, True, True)

        self.assertComplexTuplesAlmostEqual(
            expected_data0[-Ntest:], dst_data0[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data1[-Ntest:], dst_data1[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data2[-Ntest:], dst_data2[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data3[-Ntest:], dst_data3[-Ntest:], 4)

    def test_001(self):
        Ntest = 50
        dst_data0, expected_data0 = run_test(self.tb, 1, False, False)
        dst_data1, expected_data1 = run_test(self.tb, 1, False, True)
        dst_data2, expected_data2 = run_test(self.tb, 1, True, False)
        dst_data3, expected_data3 = run_test(self.tb, 1, True, True)

        self.assertComplexTuplesAlmostEqual(
            expected_data0[-Ntest:], dst_data0[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data1[-Ntest:], dst_data1[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data2[-Ntest:], dst_data2[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data3[-Ntest:], dst_data3[-Ntest:], 4)

    def test_002(self):
        Ntest = 50
        dst_data0, expected_data0 = run_test(self.tb, 2, False, False)
        dst_data1, expected_data1 = run_test(self.tb, 2, False, True)
        dst_data2, expected_data2 = run_test(self.tb, 2, True, False)
        dst_data3, expected_data3 = run_test(self.tb, 2, True, True)

        self.assertComplexTuplesAlmostEqual(
            expected_data0[-Ntest:], dst_data0[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data1[-Ntest:], dst_data1[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data2[-Ntest:], dst_data2[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data3[-Ntest:], dst_data3[-Ntest:], 4)

    def test_003(self):
        Ntest = 50
        dst_data0, expected_data0 = run_test(self.tb, 3, False, False)
        dst_data1, expected_data1 = run_test(self.tb, 3, False, True)
        dst_data2, expected_data2 = run_test(self.tb, 3, True, False)
        dst_data3, expected_data3 = run_test(self.tb, 3, True, True)

        self.assertComplexTuplesAlmostEqual(
            expected_data0[-Ntest:], dst_data0[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data1[-Ntest:], dst_data1[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data2[-Ntest:], dst_data2[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data3[-Ntest:], dst_data3[-Ntest:], 4)

    def test_004(self):
        Ntest = 50
        dst_data0, expected_data0 = run_test(self.tb, 4, False, False)
        dst_data1, expected_data1 = run_test(self.tb, 4, False, True)
        dst_data2, expected_data2 = run_test(self.tb, 4, True, False)
        dst_data3, expected_data3 = run_test(self.tb, 4, True, True)

        self.assertComplexTuplesAlmostEqual(
            expected_data0[-Ntest:], dst_data0[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data1[-Ntest:], dst_data1[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data2[-Ntest:], dst_data2[-Ntest:], 4)
        self.assertComplexTuplesAlmostEqual(
            expected_data3[-Ntest:], dst_data3[-Ntest:], 4)


if __name__ == '__main__':
    gr_unittest.run(test_pfb_decimator)



# Copyright 2016 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import pytest

from os import path
import tempfile

from grc.core.platform import Platform


def test_generator():
    # c&p form compiler code.
    grc_file = path.join(path.dirname(__file__),
                         'resources', 'test_compiler.grc')
    out_dir = tempfile.gettempdir()
    block_paths = [
        path.join(path.dirname(__file__), '../../grc/blocks'),
        path.join(path.dirname(__file__), '../../gr-blocks/grc')
    ]

    platform = Platform(
        name='GNU Radio Companion Compiler',
        prefs=None,
        version='0.0.0',
    )
    platform.build_library(block_paths)

    flow_graph = platform.make_flow_graph(grc_file)
    flow_graph.rewrite()
    flow_graph.validate()

    assert flow_graph.is_valid()

    generator = platform.Generator(
        flow_graph, path.join(path.dirname(__file__), 'resources'))
    generator.write()



#!/usr/bin/env python
#
# Copyright 2013-2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy
import pmt
from gnuradio import gr, gr_unittest, blocks
from gnuradio.gr import packet_utils


def make_tag(key, value, offset, srcid=None):
    tag = gr.tag_t()
    tag.key = pmt.string_to_symbol(key)
    tag.value = pmt.to_pmt(value)
    tag.offset = offset
    if srcid is not None:
        tag.srcid = pmt.to_pmt(srcid)
    return tag


class qa_tagged_stream_mux (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "tsb_key"

    def tearDown(self):
        self.tb = None

    def setup_data_tags(self, data):
        return packet_utils.packets_to_vectors(
            data,
            self.tsb_key
        )

    def test_1(self):
        packets0 = (
            (0, 1, 2), (5, 6), (10,), (14, 15, 16,)
        )
        packets1 = (
            (3, 4), (7, 8, 9), (11, 12, 13), (17,)
        )
        expected = [[0, 1, 2, 3, 4], [5, 6, 7, 8, 9],
                    [10, 11, 12, 13], [14, 15, 16, 17]]
        data0, tags0 = self.setup_data_tags(packets0)
        data1, tags1 = self.setup_data_tags(packets1)
        tags0.append(make_tag('spam', 42, 4))
        tags1.append(make_tag('eggs', 23, 3))
        src0 = blocks.vector_source_b(data0, tags=tags0)
        src1 = blocks.vector_source_b(data1, tags=tags1)
        tagged_stream_mux = blocks.tagged_stream_mux(
            gr.sizeof_char, self.tsb_key)
        snk = blocks.tsb_vector_sink_b(tsb_key=self.tsb_key)
        self.tb.connect(src0, (tagged_stream_mux, 0))
        self.tb.connect(src1, (tagged_stream_mux, 1))
        self.tb.connect(tagged_stream_mux, snk)
        self.tb.run()
        # Check
        self.assertEqual(expected, snk.data())
        tags = [gr.tag_to_python(x) for x in snk.tags()]
        tags = sorted([(x.offset, x.key, x.value) for x in tags])
        tags_expected = [
            (6, 'spam', 42),
            (8, 'eggs', 23),
        ]
        self.assertEqual(tags, tags_expected)

    def test_preserve_tag_head_pos(self):
        """ Test the 'preserve head position' function.
        This will add a 'special' tag to item 0 on stream 1.
        It should be on item 0 of the output stream. """
        packet_len_0 = 5
        data0 = list(range(packet_len_0))
        packet_len_1 = 3
        data1 = list(range(packet_len_1))
        mux = blocks.tagged_stream_mux(
            gr.sizeof_float,
            self.tsb_key,
            1  # Mark port 1 as carrying special tags on the head position
        )
        sink = blocks.tsb_vector_sink_f(tsb_key=self.tsb_key)
        self.tb.connect(blocks.vector_source_f(data0), blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, packet_len_0, self.tsb_key), (mux, 0))
        self.tb.connect(
            blocks.vector_source_f(
                list(
                    range(packet_len_1)), tags=(
                    make_tag(
                        'spam', 'eggs', 0),)), blocks.stream_to_tagged_stream(
                gr.sizeof_float, 1, packet_len_1, self.tsb_key), (mux, 1))
        self.tb.connect(mux, sink)
        self.tb.run()
        self.assertEqual(len(sink.data()), 1)
        self.assertEqual(sink.data()[0], data0 + data1)
        self.assertEqual(len(sink.tags()), 1)
        tag = gr.tag_to_python(sink.tags()[0])
        tag = (tag.offset, tag.key, tag.value)
        tag_expected = (0, 'spam', 'eggs')
        self.assertEqual(tag, tag_expected)


if __name__ == '__main__':
    gr_unittest.run(qa_tagged_stream_mux)



#!/usr/bin/env python
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import math
from cmath import exp, pi, log, sqrt
import numpy

from gnuradio import gr, gr_unittest, digital, blocks
from gnuradio.digital.utils import mod_codes
from gnuradio.digital import constellation, psk, qam, qamlike
import numpy as np

tested_mod_codes = (mod_codes.NO_CODE, mod_codes.GRAY_CODE)

# A list of the constellations to test.
# Each constellation is given by a 3-tuple.
# First item is a function to generate the constellation
# Second item is a dictionary of arguments for function with lists of
# possible values.
# Third item is whether differential encoding should be tested.
# Fourth item is the name of the argument to constructor that specifies
# whether differential encoding is used.


def twod_constell():
    """

    """
    points = ((1 + 0j), (0 + 1j),
              (-1 + 0j), (0 - 1j))
    rot_sym = 2
    dim = 2
    return digital.constellation_calcdist(points, [], rot_sym, dim)


def threed_constell():
    oned_points = ((1 + 0j), (0 + 1j), (-1 + 0j), (0 - 1j))
    points = []
    r4 = list(range(0, 4))
    for ia in r4:
        for ib in r4:
            for ic in r4:
                points += [oned_points[ia], oned_points[ib], oned_points[ic]]
    rot_sym = 4
    dim = 3
    return digital.constellation_calcdist(points, [], rot_sym, dim)

# A list of tuples for constellation testing.  The contents of the
# tuples are (constructor, poss_args, differential, diff_argname).


# These constellations should lock on well.
easy_constellation_info = (
    (psk.psk_constellation,
     {'m': (2, 4, 8, 16, ),
      'mod_code': tested_mod_codes, },
     True, None),
    (psk.psk_constellation,
     {'m': (2, 4, 8, 16, 32, 64),
      'mod_code': tested_mod_codes,
      'differential': (False,)},
     False, None),
    (qam.qam_constellation,
     {'constellation_points': (4,),
      'mod_code': tested_mod_codes,
      'large_ampls_to_corners': [False], },
     True, None),
    (qam.qam_constellation,
     {'constellation_points': (4, 16, 64),
      'mod_code': tested_mod_codes,
      'differential': (False,)},
     False, None),
    (digital.constellation_bpsk, {}, True, None),
    (digital.constellation_qpsk, {}, False, None),
    (digital.constellation_dqpsk, {}, True, None),
    (digital.constellation_8psk, {}, False, None),
    (twod_constell, {}, True, None),
    (threed_constell, {}, True, None),
)

# These constellations don't work nicely.
# We have a lower required error rate.
medium_constellation_info = (
    (psk.psk_constellation,
     {'m': (32, 64),
      'mod_code': tested_mod_codes, },
     True, None),
    (qam.qam_constellation,
     {'constellation_points': (16,),
      'mod_code': tested_mod_codes,
      'large_ampls_to_corners': [False, True], },
     True, None),
    (qamlike.qam32_holeinside_constellation,
     {'large_ampls_to_corners': [True]},
     True, None),
)

# These constellation are basically broken in our test
difficult_constellation_info = (
    (qam.qam_constellation,
     {'constellation_points': (64,),
      'mod_code': tested_mod_codes,
      'large_ampls_to_corners': [False, True], },
     True, None),
)


def slicer(x):
    ret = []
    for xi in x:
        if xi < 0:
            ret.append(0.0)
        else:
            ret.append(1.0)
    return ret


def tested_constellations(easy=True, medium=True, difficult=True):
    """
    Generator to produce (constellation, differential) tuples for testing purposes.
    """
    constellation_info = []
    if easy:
        constellation_info += easy_constellation_info
    if medium:
        constellation_info += medium_constellation_info
    if difficult:
        constellation_info += difficult_constellation_info
    for constructor, poss_args, differential, diff_argname in constellation_info:
        if differential:
            diff_poss = (True, False)
        else:
            diff_poss = (False,)
        poss_args = [[argname, argvalues, 0]
                     for argname, argvalues in list(poss_args.items())]
        for current_diff in diff_poss:
            # Add an index into args to keep track of current position in
            # argvalues
            while True:
                current_args = dict([(argname, argvalues[argindex])
                                     for argname, argvalues, argindex in poss_args])
                if diff_argname is not None:
                    current_args[diff_argname] = current_diff
                constellation = constructor(**current_args)
                yield (constellation, current_diff)
                for this_poss_arg in poss_args:
                    argname, argvalues, argindex = this_poss_arg
                    if argindex < len(argvalues) - 1:
                        this_poss_arg[2] += 1
                        break
                    else:
                        this_poss_arg[2] = 0
                if sum([argindex for argname, argvalues,
                        argindex in poss_args]) == 0:
                    break


class test_constellation(gr_unittest.TestCase):

    src_length = 256

    def setUp(self):
        random.seed(0)
        # Generate a list of random bits.
        self.src_data = [
            random.randint(
                0, 1) for i in range(
                0, self.src_length)]

    def tearDown(self):
        pass

    def test_normalization(self):
        rot_sym = 1
        side = 2
        width = 2
        # Test a couple of basic constellations
        for constel_points, code in (digital.psk_4_0(), digital.qam_16_0()):
            constel = digital.constellation_rect(constel_points, code, rot_sym,
                                                 side, side, width, width,
                                                 constellation.POWER_NORMALIZATION)

            points = np.array(constel.points())
            avg_power = np.sum(abs(points)**2) / len(points)
            self.assertAlmostEqual(avg_power, 1.0, 6)
            constel = digital.constellation_rect(constel_points, code, rot_sym,
                                                 side, side, width, width,
                                                 constellation.AMPLITUDE_NORMALIZATION)
            points = np.array(constel.points())
            avg_amp = np.sum(abs(points)) / len(points)
            self.assertAlmostEqual(avg_amp, 1.0, 6)

    def test_hard_decision(self):
        for constellation, differential in tested_constellations():
            if differential:
                rs = constellation.rotational_symmetry()
                rotations = [exp(i * 2 * pi * (0 + 1j) / rs)
                             for i in range(0, rs)]
            else:
                rotations = [None]
            for rotation in rotations:
                src = blocks.vector_source_b(self.src_data)
                content = mod_demod(constellation, differential, rotation)
                dst = blocks.vector_sink_b()
                self.tb = gr.top_block()
                self.tb.connect(src, content, dst)
                self.tb.run()
                data = dst.data()
                # Don't worry about cut off data for now.
                first = constellation.bits_per_symbol()
                equality = all(numpy.equal(self.src_data[first:len(data)],
                                           data[first:]))
                if not equality:
                    msg = "Constellations mismatched. " + \
                        f"{type(constellation)}; " + \
                        f"Differential? {differential}; " + \
                        f"{len(constellation.points())} " +\
                        "Constellation points: " + \
                        f"{constellation.points()};"
                    self.assertEqual(self.src_data[first:len(data)],
                                     data[first:],
                                     msg=msg)

    def test_soft_qpsk_gen(self):
        prec = 8

        c = digital.constellation_qpsk().base()

        constel = c.points()
        code = [0, 1, 2, 3]
        Es = 1.0

        c.set_npwr(Es)
        c.normalize(digital.constellation.POWER_NORMALIZATION)
        table = digital.soft_dec_table(constel, code, prec, Es)
        constel = digital.const_normalization(constel, "POWER")
        maxamp = digital.min_max_axes(constel)

        c.set_soft_dec_lut(table, prec)

        x = sqrt(2.0) / 2.0
        step = (x.real + x.real) / (2**prec - 1)
        samples = [-x - x * 1j,
                   -x + x * 1j,
                   x + x * 1j,
                   x - x * 1j,
                   (-x + 128 * step) + (-x + 128 * step) * 1j,
                   (-x + 64 * step) + (-x + 64 * step) * 1j,
                   (-x + 64 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 64 * step) * 1j,
                   ]

        y_python_raw_calc = []
        y_python_gen_calc = []
        y_python_table = []
        y_cpp_raw_calc = []
        y_cpp_table = []
        for sample in samples:
            y_python_raw_calc += slicer(
                digital.calc_soft_dec(
                    sample, constel, code))
            y_python_gen_calc += slicer(digital.sd_psk_4_0(sample, Es))
            y_python_table += slicer(
                digital.calc_soft_dec_from_table(
                    sample, table, prec, maxamp))

            y_cpp_raw_calc += c.calc_soft_dec(sample)
            y_cpp_table += c.soft_decision_maker(sample)

        self.assertFloatTuplesAlmostEqual(
            y_python_raw_calc, y_python_gen_calc, 3)
        self.assertFloatTuplesAlmostEqual(y_python_gen_calc, y_python_table, 1)
        self.assertFloatTuplesAlmostEqual(y_cpp_raw_calc, y_cpp_table, 1)

    def test_soft_qpsk_calc(self):
        prec = 8
        constel, code = digital.psk_4_0()

        rot_sym = 1
        side = 2
        width = 2
        c = digital.constellation_rect(constel, code, rot_sym,
                                       side, side, width, width)

        # Get max energy/symbol in constellation
        constel = c.points()
        Es = max([abs(constel_i) for constel_i in constel])

        table = digital.soft_dec_table(constel, code, prec)
        c.gen_soft_dec_lut(prec)

        x = sqrt(2.0) / 2.0
        step = (x.real + x.real) / (2**prec - 1)
        samples = [-x - x * 1j,
                   -x + x * 1j,
                   x + x * 1j,
                   x - x * 1j,
                   (-x + 128 * step) + (-x + 128 * step) * 1j,
                   (-x + 64 * step) + (-x + 64 * step) * 1j,
                   (-x + 64 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 64 * step) * 1j,
                   ]

        y_python_raw_calc = []
        y_python_table = []
        y_cpp_raw_calc = []
        y_cpp_table = []
        for sample in samples:
            y_python_raw_calc += slicer(
                digital.calc_soft_dec(
                    sample, constel, code))
            y_python_table += slicer(
                digital.calc_soft_dec_from_table(
                    sample, table, prec, Es))

            y_cpp_raw_calc += slicer(c.calc_soft_dec(sample))
            y_cpp_table += slicer(c.soft_decision_maker(sample))

        self.assertEqual(y_python_raw_calc, y_python_table)
        self.assertEqual(y_cpp_raw_calc, y_cpp_table)

    def test_soft_qam16_calc(self):
        prec = 8
        constel, code = digital.qam_16_0()

        rot_sym = 1
        side = 2
        width = 2
        c = digital.constellation_rect(constel, code, rot_sym,
                                       side, side, width, width)

        # Get max energy/symbol in constellation
        constel = c.points()
        Es = 1.0
        padding = 2

        table = digital.soft_dec_table(constel, code, prec)
        c.gen_soft_dec_lut(prec)

        x = sqrt(2.0) / 2.0
        step = (x.real + x.real) / (2**prec - 1)
        samples = [-x - x * 1j,
                   -x + x * 1j,
                   x + x * 1j,
                   x - x * 1j,
                   (-x + 128 * step) + (-x + 128 * step) * 1j,
                   (-x + 64 * step) + (-x + 64 * step) * 1j,
                   (-x + 64 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 192 * step) * 1j,
                   (-x + 192 * step) + (-x + 64 * step) * 1j,
                   ]

        y_python_raw_calc = []
        y_python_table = []
        y_cpp_raw_calc = []
        y_cpp_table = []
        for sample in samples:
            y_python_raw_calc += slicer(
                digital.calc_soft_dec(
                    sample, constel, code))
            y_python_table += slicer(
                digital.calc_soft_dec_from_table(
                    sample, table, prec, Es))

            y_cpp_raw_calc += slicer(c.calc_soft_dec(sample))
            y_cpp_table += slicer(c.soft_decision_maker(sample))

        self.assertFloatTuplesAlmostEqual(y_python_raw_calc, y_python_table, 3)
        self.assertFloatTuplesAlmostEqual(y_cpp_raw_calc, y_cpp_table, 3)


class mod_demod(gr.hier_block2):
    def __init__(self, constellation, differential, rotation):
        if constellation.arity() > 256:
            # If this becomes limiting some of the blocks should be generalised so
            # that they can work with shorts and ints as well as chars.
            raise ValueError(
                "Constellation cannot contain more than 256 points.")

        gr.hier_block2.__init__(self, "mod_demod",
                                gr.io_signature(
                                    1, 1, gr.sizeof_char),       # Input signature
                                gr.io_signature(1, 1, gr.sizeof_char))       # Output signature

        arity = constellation.arity()

        # TX
        self.constellation = constellation
        self.differential = differential
        import weakref
        self.blocks = [weakref.proxy(self)]
        # We expect a stream of unpacked bits.
        # First step is to pack them.
        self.blocks.append(blocks.unpacked_to_packed_bb(1, gr.GR_MSB_FIRST))
        # Second step we unpack them such that we have k bits in each byte where
        # each constellation symbol hold k bits.
        self.blocks.append(
            blocks.packed_to_unpacked_bb(self.constellation.bits_per_symbol(),
                                         gr.GR_MSB_FIRST))
        # Apply any pre-differential coding
        # Gray-coding is done here if we're also using differential coding.
        if self.constellation.apply_pre_diff_code():
            self.blocks.append(
                digital.map_bb(
                    self.constellation.pre_diff_code()))
        # Differential encoding.
        if self.differential:
            self.blocks.append(digital.diff_encoder_bb(arity))
        # Convert to constellation symbols.
        self.blocks.append(
            digital.chunks_to_symbols_bc(
                self.constellation.points(),
                self.constellation.dimensionality()))
        # CHANNEL
        # Channel just consists of a rotation to check differential coding.
        if rotation is not None:
            self.blocks.append(blocks.multiply_const_cc(rotation))

        # RX
        # Convert the constellation symbols back to binary values.
        self.blocks.append(
            digital.constellation_decoder_cb(
                self.constellation.base()))
        # Differential decoding.
        if self.differential:
            self.blocks.append(digital.diff_decoder_bb(arity))
        # Decode any pre-differential coding.
        if self.constellation.apply_pre_diff_code():
            self.blocks.append(digital.map_bb(
                mod_codes.invert_code(self.constellation.pre_diff_code())))
        # unpack the k bit vector into a stream of bits
        self.blocks.append(blocks.unpack_k_bits_bb(
            self.constellation.bits_per_symbol()))
        # connect to block output
        check_index = len(self.blocks)
        self.blocks = self.blocks[:check_index]
        self.blocks.append(weakref.proxy(self))

        self.connect(*self.blocks)


if __name__ == '__main__':
    gr_unittest.run(test_constellation)



#!/usr/bin/env python
#
# Copyright 2011,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import random
import cmath
import time

from gnuradio import gr, gr_unittest, filter, digital, blocks


class test_pfb_clock_sync(gr_unittest.TestCase):

    def setUp(self):
        random.seed(0)
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test01(self):
        # Test BPSK sync
        excess_bw = 0.35

        sps = 4
        loop_bw = cmath.pi / 100.0
        nfilts = 32
        init_phase = nfilts / 2
        max_rate_deviation = 0.5
        osps = 1

        ntaps = 11 * int(sps * nfilts)
        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw, ntaps)

        self.test = digital.pfb_clock_sync_ccf(sps, loop_bw, taps,
                                               nfilts, init_phase,
                                               max_rate_deviation,
                                               osps)

        data = 10000 * [complex(1, 0), complex(-1, 0)]
        self.src = blocks.vector_source_c(data, False)

        # pulse shaping interpolation filter
        rrc_taps = filter.firdes.root_raised_cosine(
            nfilts,          # gain
            nfilts,          # sampling rate based on 32 filters in resampler
            1.0,             # symbol rate
            excess_bw,       # excess bandwidth (roll-off factor)
            ntaps)
        self.rrc_filter = filter.pfb_arb_resampler_ccf(sps, rrc_taps)

        self.snk = blocks.vector_sink_c()

        self.tb.connect(self.src, self.rrc_filter, self.test, self.snk)
        self.tb.run()

        expected_result = 10000 * [complex(1, 0), complex(-1, 0)]
        dst_data = self.snk.data()

        # Only compare last Ncmp samples
        Ncmp = 1000
        len_e = len(expected_result)
        len_d = len(dst_data)
        expected_result = expected_result[len_e - Ncmp:]
        dst_data = dst_data[len_d - Ncmp:]

        # for e,d in zip(expected_result, dst_data):
        #    print e, d

        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 1)

    def test02(self):
        # Test real BPSK sync
        excess_bw = 0.35

        sps = 4
        loop_bw = cmath.pi / 100.0
        nfilts = 32
        init_phase = nfilts / 2
        max_rate_deviation = 0.5
        osps = 1

        ntaps = 11 * int(sps * nfilts)
        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw, ntaps)

        self.test = digital.pfb_clock_sync_fff(sps, loop_bw, taps,
                                               nfilts, init_phase,
                                               max_rate_deviation,
                                               osps)

        data = 10000 * [1, -1]
        self.src = blocks.vector_source_f(data, False)

        # pulse shaping interpolation filter
        rrc_taps = filter.firdes.root_raised_cosine(
            nfilts,          # gain
            nfilts,          # sampling rate based on 32 filters in resampler
            1.0,             # symbol rate
            excess_bw,       # excess bandwidth (roll-off factor)
            ntaps)
        self.rrc_filter = filter.pfb_arb_resampler_fff(sps, rrc_taps)

        self.snk = blocks.vector_sink_f()

        self.tb.connect(self.src, self.rrc_filter, self.test, self.snk)
        self.tb.run()

        expected_result = 10000 * [1, -1]
        dst_data = self.snk.data()

        # Only compare last Ncmp samples
        Ncmp = 1000
        len_e = len(expected_result)
        len_d = len(dst_data)
        expected_result = expected_result[len_e - Ncmp:]
        dst_data = dst_data[len_d - Ncmp:]

        # for e,d in zip(expected_result, dst_data):
        #    print e, d

        self.assertFloatTuplesAlmostEqual(expected_result, dst_data, 1)

    def test03(self):
        # Test resting of taps
        excess_bw0 = 0.35
        excess_bw1 = 0.22

        sps = 4
        loop_bw = cmath.pi / 100.0
        nfilts = 32
        init_phase = nfilts / 2
        max_rate_deviation = 0.5
        osps = 1

        ntaps = 11 * int(sps * nfilts)
        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw0, ntaps)

        self.test = digital.pfb_clock_sync_ccf(sps, loop_bw, taps,
                                               nfilts, init_phase,
                                               max_rate_deviation,
                                               osps)

        self.src = blocks.null_source(gr.sizeof_gr_complex)
        self.snk = blocks.null_sink(gr.sizeof_gr_complex)

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.start()
        time.sleep(0.1)

        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw1, ntaps)

        self.test.update_taps(taps)

        self.tb.stop()
        self.tb.wait()

    def test03_f(self):
        # Test resting of taps
        excess_bw0 = 0.35
        excess_bw1 = 0.22

        sps = 4
        loop_bw = cmath.pi / 100.0
        nfilts = 32
        init_phase = nfilts / 2
        max_rate_deviation = 0.5
        osps = 1

        ntaps = 11 * int(sps * nfilts)
        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw0, ntaps)

        self.test = digital.pfb_clock_sync_fff(sps, loop_bw, taps,
                                               nfilts, init_phase,
                                               max_rate_deviation,
                                               osps)

        self.src = blocks.null_source(gr.sizeof_float)
        self.snk = blocks.null_sink(gr.sizeof_float)

        self.tb.connect(self.src, self.test, self.snk)
        self.tb.start()
        time.sleep(0.1)

        taps = filter.firdes.root_raised_cosine(nfilts, nfilts * sps,
                                                1.0, excess_bw1, ntaps)

        self.test.update_taps(taps)

        self.tb.stop()
        self.tb.wait()


if __name__ == '__main__':
    gr_unittest.run(test_pfb_clock_sync)



# pip install selenium
# I had to also do pip install --upgrade requests
# Download geckodriver from here https://github.com/mozilla/geckodriver/releases and put it in Downloads
# sudo chmod +x geckodriver
# export PATH=$PATH:/home/marc/Downloads (or wherever you put it)

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
import time
from html.parser import HTMLParser
import os

# Settings
pages_to_save = ['Handling Flowgraphs',
                 'Types of Blocks',
                 'Metadata Information',
                 'Stream Tags',
                 'Block Thread Affinity and Priority',
                 'Configuration Files',
                 'VOLK Guide',
                 'Polymorphic Types (PMTs)',
                 'Message Passing',
                 'QT GUI',
                 'Logging',
                 'Performance Counters',
                 'Tagged Stream Blocks',
                 'Polyphase Filterbanks']

# set up web driver
# dir that contains geckodriver
driver = webdriver.Firefox('/home/marc/Downloads/geckodriver')
print("STARTING")
for page_name in pages_to_save:
    print("Processing", page_name)
    driver.get("https://wiki.gnuradio.org/index.php/Special:Export")

    # fill in text box
    text_area = driver.find_element_by_xpath("//*[@name='pages']")
    text_area.send_keys(page_name)

    # uncheck "save as file" box
    check_box = driver.find_element_by_xpath("//*[@name='wpDownload']")
    check_box.click()

    # hit Export
    submit_button = driver.find_element_by_xpath("//*[@value='Export']")
    submit_button.click()

    # get HTML of new page
    raw_html = driver.page_source
    start_index = raw_html.find('<page>')
    cropped_html = raw_html[start_index:]

    # save text to file
    h = HTMLParser()
    # makes it so stuff like &gt shows up as a greater than sign
    cropped_html_text = h.unescape(cropped_html)
    text_file = open("(exported from wiki) " + page_name + ".txt", "w")
    text_file.write(cropped_html_text)
    text_file.close()

driver.close()
print("DONE")



#!/usr/bin/env python
#
# Copyright 2008,2010,2012,2013 Free Software Foundation, Inc.
# Copyright 2024 Daniel Estevez <daniel@destevez.net>
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, digital, blocks
from gnuradio.digital.utils import lfsr_args
import numpy as np
import pmt

# See gr-digital/lib/additive_scrambler_bb_impl.cc for reference.


def additive_scramble_lfsr(mask, seed, reglen, bpb, data):
    l = digital.lfsr(mask, seed, reglen)
    out = []
    for d in data:
        scramble_word = 0
        for i in range(0, bpb):
            scramble_word ^= l.next_bit() << i
        out.append(d ^ scramble_word)
    return out


class test_scrambler(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_lfsr_002(self):
        _a = lfsr_args(1, 51, 3, 0)
        l = digital.lfsr(*_a)
        seq = [l.next_bit() for _ in range(2**10)]
        reg = np.zeros(52, np.int8)
        reg[::-1][(51, 3, 0), ] = 1
        res = (np.convolve(seq, reg) % 2)
        self.assertEqual(sum(res[52:-52]), 0, msg="LRS not generated properly")

    def test_scrambler_descrambler_001(self):
        src_data = np.random.randint(0, 2, 500, dtype=np.int8)
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.scrambler_bb(
            *lfsr_args(0b1, 7, 2, 0))     # p(x) = x^7 + x^2 + 1
        # we can use any seed here, it is descrambling.
        descrambler = digital.descrambler_bb(*lfsr_args(0b111, 7, 2, 0))
        m_tap = blocks.vector_sink_b()
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.connect(scrambler, m_tap)
        self.tb.run()
        # skip garbage during synchronization
        self.assertEqual(src_data[:-7].tolist(), dst.data()[7:])
        self.assertEqual(tuple(np.convolve(m_tap.data(), [1, 0, 0, 0, 0, 1, 0, 1]) % 2)[7:-10],
                         tuple(src_data[:-10]))  # manual descrambling test

    def test_scrambler_descrambler_002(self):
        _a = lfsr_args(0b1, 51, 6, 0)  # p(x) = x^51+x^6+1
        src_data = np.random.randint(0, 2, 1000, dtype=np.int8)
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.scrambler_bb(*_a)
        m_tap = blocks.vector_sink_b()
        descrambler = digital.descrambler_bb(*_a)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.connect(scrambler, m_tap)
        self.tb.run()
        # skip garbage during synchronization
        self.assertTrue(np.all(src_data[:-51] == dst.data()[51:]))
        reg = np.zeros(52, np.int8)
        reg[::-1][(51, 6, 0), ] = 1
        self.assertTrue(np.all(np.convolve(m_tap.data(), reg)[51:-60] % 2 ==
                               src_data[:-60]))  # manual descrambling test

    def test_scrambler_descrambler_003(self):
        src_data = np.random.randint(0, 2, 1000, dtype=np.int8)
        src = blocks.vector_source_b(src_data, False)
        # this is the product of the other two
        scrambler = digital.scrambler_bb(*lfsr_args(1, 12, 10, 3, 2, 0))
        descrambler1 = digital.descrambler_bb(*lfsr_args(1, 5, 3, 0))
        descrambler2 = digital.descrambler_bb(*lfsr_args(1, 7, 2, 0))
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler1, descrambler2, dst)
        self.tb.run()
        # skip garbage during synchronization
        self.assertTrue(np.all(src_data[:-12] == dst.data()[12:]))

    def test_additive_scrambler_001(self):
        _a = lfsr_args(1, 51, 3, 0)  # i p(x) = x^51+x^3+1, seed 0x1
        src_data = np.random.randint(0, 2, 1000, dtype=np.int8).tolist()
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(*_a)
        descrambler = digital.additive_scrambler_bb(*_a)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.run()
        self.assertEqual(tuple(src_data), tuple(dst.data()))

    def test_additive_soft_symbols_001(self):
        _a = lfsr_args(1, 51, 3, 0)  # i p(x) = x^51+x^3+1, seed 0x1
        src_data = np.random.randn(1000).tolist()
        src = blocks.vector_source_f(src_data, False)
        scrambler = digital.additive_scrambler_ff(*_a)
        descrambler = digital.additive_scrambler_ff(*_a)
        dst = blocks.vector_sink_f()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.run()
        self.assertFloatTuplesAlmostEqual(tuple(src_data), tuple(dst.data()),
                                          places=6)

    def test_additive_scrambler_002(self):
        _a = lfsr_args(1, 51, 3, 0)  # i p(x) = x^51+x^3+1, seed 0x1
        src_data = [1, ] * 1000
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(*_a)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, dst)
        self.tb.run()
        reg = np.zeros(52, np.int8)
        reg[::-1][(51, 3, 0), ] = 1
        res = (np.convolve(dst.data(), reg) % 2)[52:-52]
        self.assertEqual(len(res), sum(res))  # when convolved with mask,
        # after sync, only 1's would be returned.

    def test_scrambler_descrambler(self):
        src_data = [1, ] * 1000
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.scrambler_bb(
            0x8a, 0x7F, 7)     # CCSDS 7-bit scrambler
        descrambler = digital.descrambler_bb(0x8a, 0x7F, 7)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.run()
        # skip garbage during synchronization
        self.assertEqual(src_data[:-8], dst.data()[8:])

    def test_additive_scrambler(self):
        src_data = [1, ] * 1000
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7)
        descrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.run()
        self.assertEqual(src_data, dst.data())

    def test_additive_scrambler_reset(self):
        src_data = [1, ] * 200
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7, 50)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, dst)
        self.tb.run()
        output = dst.data()
        self.assertEqual(output[:50] * 4, output)

    def test_additive_scrambler_reset_3bpb(self):
        src_data = [5, ] * 200
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7, 50, 3)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, dst)
        self.tb.run()
        output = dst.data()
        self.assertEqual(output[:50] * 4, output)

    def test_additive_scrambler_tags(self):
        src_data = [1, ] * 1000
        src = blocks.vector_source_b(src_data, False)
        scrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7, 100)
        descrambler = digital.additive_scrambler_bb(0x8a, 0x7f, 7, 100)
        reset_tag_key = 'reset_lfsr'
        reset_tag1 = gr.tag_t()
        reset_tag1.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag1.offset = 17
        reset_tag2 = gr.tag_t()
        reset_tag2.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag2.offset = 110
        reset_tag3 = gr.tag_t()
        reset_tag3.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag3.offset = 523
        src = blocks.vector_source_b(
            src_data, False, 1, (reset_tag1, reset_tag2, reset_tag3))
        scrambler = digital.additive_scrambler_bb(
            0x8a, 0x7f, 7, 100, 1, reset_tag_key)
        descrambler = digital.additive_scrambler_bb(
            0x8a, 0x7f, 7, 100, 1, reset_tag_key)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, descrambler, dst)
        self.tb.run()
        self.assertEqual(src_data, dst.data())

    def test_additive_scrambler_tags_oneway(self):
        src_data = [x for x in range(0, 10)]
        reset_tag_key = 'reset_lfsr'
        reset_tag1 = gr.tag_t()
        reset_tag1.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag1.offset = 0
        reset_tag2 = gr.tag_t()
        reset_tag2.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag2.offset = 10
        reset_tag3 = gr.tag_t()
        reset_tag3.key = pmt.string_to_symbol(reset_tag_key)
        reset_tag3.offset = 20
        src = blocks.vector_source_b(
            src_data * 3, False, 1, (reset_tag1, reset_tag2, reset_tag3))
        scrambler = digital.additive_scrambler_bb(
            0x8a, 0x7f, 7, 0, 8, reset_tag_key)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, scrambler, dst)
        self.tb.run()
        expected_data = additive_scramble_lfsr(0x8a, 0x7f, 7, 8, src_data)
        self.assertEqual(expected_data * 3, dst.data())


if __name__ == '__main__':
    gr_unittest.run(test_scrambler)



#!/usr/bin/env python
#
# Copyright 2023 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import blocks
from gnuradio import gr_unittest
from gnuradio import gr
import pmt
import time


class test_msg_pair_to_var(gr_unittest.TestCase):
    def setUp(self):
        self.backend = gr.dictionary_logger_backend()
        gr.logging().add_default_sink(self.backend)
        self.tb = gr.top_block(catch_exceptions=False)

    def tearDown(self):
        self.tb = None

    def test_calling(self):
        canary = 0xDEAFBEEF
        self.test_val = 0

        def test_f(value: int = 0):
            self.test_val = value

        dut = blocks.msg_pair_to_var(test_f)
        test_msg = pmt.cons(pmt.intern("foo"), pmt.from_uint64(canary))
        src = blocks.message_strobe(test_msg, 50)
        self.tb.msg_connect(*(src, pmt.intern("strobe")),
                            *(dut, blocks.msg_pair_to_var.IN_PORT))

        start_time = time.monotonic_ns()
        self.tb.start()

        # Sleep for at most a second
        for counter in range(20):
            time.sleep(50e-3)
            if self.test_val == canary:
                break
        self.tb.stop()
        self.tb.wait()
        stopped_time = time.monotonic_ns()
        timecount = round((stopped_time - start_time) * 1e-6)

        self.assertEqual(
            self.test_val,
            canary,
            f"Test value {hex(self.test_val)} not found to be canary {hex(canary)} within {timecount} ms",
        )

    def test_exceptions(self):
        lark = 0xDEAFBEEF
        nightingale = 0xDEAFB16D
        self.test_val = False

        def test_f(value: int = 0):
            if value == nightingale and value != lark:
                self.test_val = True
                raise Exception("It was the nightingale, and not the lark")

        dut = blocks.msg_pair_to_var(test_f)
        test_msg = pmt.cons(pmt.intern("birb"), pmt.from_uint64(nightingale))
        src = blocks.message_strobe(test_msg, 50)
        self.tb.msg_connect(
            *(src, pmt.intern("strobe")), *(dut, blocks.msg_pair_to_var.IN_PORT)
        )

        start_time = time.monotonic_ns()
        self.tb.start()

        # Sleep for at most a second
        for counter in range(19):
            time.sleep(50e-3)
            if self.test_val:
                break
        # some time for Exception bubbling, which should happen without yields
        time.sleep(50e-3)
        self.tb.stop()
        self.tb.wait()
        stopped_time = time.monotonic_ns()
        timecount = round((stopped_time - start_time) * 1e-6)

        logged = self.backend.get_map()
        self.assertIn("msg_pair_to_var", logged, f"should have logged; didn't within {timecount} ms.")
        log_entries = logged["msg_pair_to_var"]
        for timestamp, entry in log_entries:
            self.assertRegex(
                entry,
                r"Error when calling <function .*\.test_exceptions\..*.test_f.*> with 3736056173" +
                r".*reason: It was the nightingale, and not the lark.*",
                f"Log entry '{entry}' from {timestamp} doesn't contain Shakespeare",
            )


if __name__ == "__main__":
    gr_unittest.run(test_msg_pair_to_var)



#
# Copyright 2014 Free Software Foundation, Inc.
# Copyright 2021 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import time

from gnuradio import gr_unittest, blocks, gr
from gnuradio.gr.hier_block2 import _multiple_endpoints, _optional_endpoints
import pmt


class test_hblk(gr.hier_block2):
    def __init__(self, io_sig=1 * [gr.sizeof_gr_complex], ndebug=2):
        # parent constructor
        gr.hier_block2.__init__(
            self, "test_hblk", gr.io_signature(
                len(io_sig), len(io_sig), io_sig[0]), gr.io_signature(
                0, 0, 0))

        self.message_port_register_hier_in("msg_in")

        # Internal Stream Blocks
        self.vsnk = blocks.vector_sink_c()

        # Internal Msg Blocks
        self.blks = []
        for i in range(0, ndebug):
            self.blks.append(blocks.message_debug())

        # Set up internal connections
        self.connect(self, self.vsnk)
        for blk in self.blks:
            self.msg_connect(self, "msg_in", blk, "print")


class test_hier_block2(gr_unittest.TestCase):

    def setUp(self):
        self.call_log = []
        self.Block = type("Block", (), {"to_basic_block": lambda bl: bl})

    def test_f(self, *args):
        """test doc"""
        self.call_log.append(args)

    multi = _multiple_endpoints(test_f)
    opt = _optional_endpoints(test_f)

    def test_000(self):
        self.assertEqual(self.multi.__doc__, "test doc")
        self.assertEqual(self.multi.__name__, "test_f")

    def test_001(self):
        b = self.Block()
        self.multi(b)
        self.assertEqual((b,), self.call_log[0])

    def test_002(self):
        b1, b2 = self.Block(), self.Block()
        self.multi(b1, b2)
        self.assertEqual([(b1, 0, b2, 0)], self.call_log)

    def test_003(self):
        b1, b2 = self.Block(), self.Block()
        self.multi((b1, 1), (b2, 2))
        self.assertEqual([(b1, 1, b2, 2)], self.call_log)

    def test_004(self):
        b1, b2, b3, b4 = [self.Block()] * 4
        self.multi(b1, (b2, 5), b3, (b4, 0))
        expected = [
            (b1, 0, b2, 5),
            (b2, 5, b3, 0),
            (b3, 0, b4, 0),
        ]
        self.assertEqual(expected, self.call_log)

    def test_005(self):
        with self.assertRaises(ValueError):
            self.multi((self.Block(), 5))

    def test_006(self):
        with self.assertRaises(ValueError):
            self.multi(self.Block(), (5, 5))

    def test_007(self):
        b1, b2 = self.Block(), self.Block()
        self.opt(b1, "in", b2, "out")
        self.assertEqual([(b1, "in", b2, "out")], self.call_log)

    def test_008(self):
        f, b1, b2 = self.multi, self.Block(), self.Block()
        self.opt((b1, "in"), (b2, "out"))
        self.assertEqual([(b1, "in", b2, "out")], self.call_log)

    def test_009(self):
        with self.assertRaises(ValueError):
            self.multi(self.Block(), 5)

    def test_010_end_with_head(self):
        import math
        exp = 1j * 440 / 44100
        src = blocks.vector_source_c([math.e**(exp * n) for n in range(10**6)])
        head = blocks.head(gr.sizeof_gr_complex, 1000)
        test = test_hblk([gr.sizeof_gr_complex], 0)
        tb = gr.top_block()
        tb.connect(src, head, test)
        tb.run()

    def test_011_test_message_connect(self):
        import math
        exp = 1j * 440 / 44100
        src = blocks.vector_source_c([math.e**(exp * n) for n in range(10**6)])
        strobe = blocks.message_strobe(pmt.PMT_NIL, 100)
        head = blocks.head(gr.sizeof_gr_complex, 1000)
        test = test_hblk([gr.sizeof_gr_complex], 1)
        tb = gr.top_block()
        tb.connect(src, head, test)
        tb.msg_connect(strobe, "strobe", test, "msg_in")
        tb.start()
        time.sleep(0.5)
        tb.stop()
        tb.wait()

    def test_012(self):
        import math
        exp = 1j * 440 / 44100
        src = blocks.vector_source_c([math.e**(exp * n) for n in range(10**6)])
        strobe = blocks.message_strobe(pmt.PMT_NIL, 100)
        head = blocks.head(gr.sizeof_gr_complex, 1000)
        test = test_hblk([gr.sizeof_gr_complex], 16)
        tb = gr.top_block()
        tb.connect(src, head, test)
        tb.msg_connect(strobe, "strobe", test, "msg_in")
        tb.start()
        time.sleep(0.5)
        tb.stop()
        tb.wait()


if __name__ == '__main__':
    gr_unittest.run(test_hier_block2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 Jacob Gilbert.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import pdu


class qa_pdu_lambda(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_smoketest(self):
        # FIXME: Test will fail until you pass sensible arguments to the constructor
        instance = pdu.pdu_lambda(lambda uvec: uvec * 10, False)

    def test_001_descriptive_test_name(self):
        # set up fg
        self.tb.run()
        # check data


if __name__ == '__main__':
    gr_unittest.run(qa_pdu_lambda)



#!/usr/bin/env python
#
# Copyright 2011-2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import ctypes


class test_endian_swap(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [1, 2, 3, 4]
        expected_result = [256, 512, 768, 1024]

        src = blocks.vector_source_s(src_data)
        op = blocks.endian_swap(2)
        dst = blocks.vector_sink_s()

        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = list(dst.data())

        self.assertEqual(expected_result, result_data)

    def test_002(self):

        src_data = [1, 2, 3, 4]
        expected_result = [16777216, 33554432, 50331648, 67108864]

        src = blocks.vector_source_i(src_data)
        op = blocks.endian_swap(4)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op, dst)
        self.tb.run()
        result_data = list(dst.data())

        self.assertEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_endian_swap)



#!/usr/bin/env python
#
# Copyright 2004,2007,2008,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_boolean_operators (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def help_ss(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_s(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_s()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_bb(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_b(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_b()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ii(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_i(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_i()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def test_xor_ss(self):
        src1_data = [1, 2, 3, 0x5004, 0x1150]
        src2_data = [8, 2, 1, 0x0508, 0x1105]
        expected_result = [9, 0, 2, 0x550C, 0x0055]
        op = blocks.xor_ss()
        self.help_ss((src1_data, src2_data),
                     expected_result, op)

    def test_xor_bb(self):
        src1_data = [1, 2, 3, 4, 0x50]
        src2_data = [8, 2, 1, 8, 0x05]
        expected_result = [9, 0, 2, 0xC, 0x55]
        op = blocks.xor_bb()
        self.help_bb((src1_data, src2_data),
                     expected_result, op)

    def test_xor_ii(self):
        src1_data = [1, 2, 3, 0x5000004, 0x11000050]
        src2_data = [8, 2, 1, 0x0500008, 0x11000005]
        expected_result = [9, 0, 2, 0x550000C, 0x00000055]
        op = blocks.xor_ii()
        self.help_ii((src1_data, src2_data),
                     expected_result, op)

    def test_and_ss(self):
        src1_data = [1, 2, 3, 0x5004, 0x1150]
        src2_data = [8, 2, 1, 0x0508, 0x1105]
        expected_result = [0, 2, 1, 0x0000, 0x1100]
        op = blocks.and_ss()
        self.help_ss((src1_data, src2_data),
                     expected_result, op)

    def test_and_bb(self):
        src1_data = [1, 2, 2, 3, 0x04, 0x50]
        src2_data = [8, 2, 2, 1, 0x08, 0x05]
        src3_data = [8, 2, 1, 1, 0x08, 0x05]
        expected_result = [0, 2, 0, 1, 0x00, 0x00]
        op = blocks.and_bb()
        self.help_bb((src1_data, src2_data, src3_data),
                     expected_result, op)

    def test_and_ii(self):
        src1_data = [1, 2, 3, 0x50005004, 0x11001150]
        src2_data = [8, 2, 1, 0x05000508, 0x11001105]
        expected_result = [0, 2, 1, 0x00000000, 0x11001100]
        op = blocks.and_ii()
        self.help_ii((src1_data, src2_data),
                     expected_result, op)

    def test_and_const_ss(self):
        src_data = [1, 2, 3, 0x5004, 0x1150]
        expected_result = [0, 2, 2, 0x5000, 0x1100]
        src = blocks.vector_source_s(src_data)
        op = blocks.and_const_ss(0x55AA)
        dst = blocks.vector_sink_s()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(dst.data(), expected_result)

    def test_and_const_bb(self):
        src_data = [1, 2, 3, 0x50, 0x11]
        expected_result = [0, 2, 2, 0x00, 0x00]
        src = blocks.vector_source_b(src_data)
        op = blocks.and_const_bb(0xAA)
        dst = blocks.vector_sink_b()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(dst.data(), expected_result)

    def test_and_const_ii(self):
        src_data = [1, 2, 3, 0x5004, 0x1150]
        expected_result = [0, 2, 2, 0x5000, 0x1100]
        src = blocks.vector_source_i(src_data)
        op = blocks.and_const_ii(0x55AA)
        dst = blocks.vector_sink_i()
        self.tb.connect(src, op, dst)
        self.tb.run()
        self.assertEqual(dst.data(), expected_result)

    def test_or_ss(self):
        src1_data = [1, 2, 3, 0x5004, 0x1150]
        src2_data = [8, 2, 1, 0x0508, 0x1105]
        expected_result = [9, 2, 3, 0x550C, 0x1155]
        op = blocks.or_ss()
        self.help_ss((src1_data, src2_data),
                     expected_result, op)

    def test_or_bb(self):
        src1_data = [1, 2, 2, 3, 0x04, 0x50]
        src2_data = [8, 2, 2, 1, 0x08, 0x05]
        src3_data = [8, 2, 1, 1, 0x08, 0x05]
        expected_result = [9, 2, 3, 3, 0x0C, 0x55]
        op = blocks.or_bb()
        self.help_bb((src1_data, src2_data, src3_data),
                     expected_result, op)

    def test_or_ii(self):
        src1_data = [1, 2, 3, 0x50005004, 0x11001150]
        src2_data = [8, 2, 1, 0x05000508, 0x11001105]
        expected_result = [9, 2, 3, 0x5500550C, 0x11001155]
        op = blocks.or_ii()
        self.help_ii((src1_data, src2_data),
                     expected_result, op)

    def test_not_ss(self):
        src1_data = [1, 2, 3, 0x5004, 0x1150]
        expected_result = [~1, ~2, ~3, ~0x5004, ~0x1150]
        op = blocks.not_ss()
        self.help_ss((((src1_data),)),
                     expected_result, op)

    def test_not_bb(self):
        src1_data = [1, 2, 2, 3, 0x04, 0x50]
        expected_result = [0xFE, 0xFD, 0xFD, 0xFC, 0xFB, 0xAF]
        op = blocks.not_bb()
        self.help_bb(((src1_data), ),
                     expected_result, op)

    def test_not_ii(self):
        src1_data = [1, 2, 3, 0x50005004, 0x11001150]
        expected_result = [~1, ~2, ~3, ~0x50005004, ~0x11001150]
        op = blocks.not_ii()
        self.help_ii(((src1_data),),
                     expected_result, op)


if __name__ == '__main__':
    gr_unittest.run(test_boolean_operators)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import blocks, analog
import numpy as np


class qa_random_uniform_source(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_byte(self):
        minimum = 0
        maximum = 5
        seed = 3
        n_items = 10000
        rnd_src = analog.random_uniform_source_b(minimum, maximum, seed)
        head = blocks.head(1, n_items)
        snk = blocks.vector_sink_b(1)
        self.tb.connect(rnd_src, head, snk)
        # set up fg
        self.tb.run()
        # check data
        res = snk.data()
        self.assertGreaterEqual(minimum, np.min(res))
        self.assertLess(np.max(res), maximum)

    def test_002_short(self):
        minimum = 42
        maximum = 1025
        seed = 3
        n_items = 10000
        rnd_src = analog.random_uniform_source_s(minimum, maximum, seed)
        head = blocks.head(2, n_items)
        snk = blocks.vector_sink_s(1)
        self.tb.connect(rnd_src, head, snk)
        # set up fg
        self.tb.run()
        # check data
        res = snk.data()
        self.assertGreaterEqual(minimum, np.min(res))
        self.assertLess(np.max(res), maximum)

    def test_003_int(self):
        minimum = 2 ** 12 - 2
        maximum = 2 ** 17 + 5
        seed = 3
        n_items = 10000
        rnd_src = analog.random_uniform_source_i(minimum, maximum, seed)
        head = blocks.head(4, n_items)
        snk = blocks.vector_sink_i(1)
        self.tb.connect(rnd_src, head, snk)
        # set up fg
        self.tb.run()
        # check data
        res = snk.data()
        # plt.hist(res)
        # plt.show()

        self.assertGreaterEqual(np.min(res), minimum)
        self.assertLess(np.max(res), maximum)


if __name__ == '__main__':
    gr_unittest.run(qa_random_uniform_source)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import time
from gnuradio import gr, gr_unittest, blocks


class test_sample_and_hold(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = 10 * [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 8, 7, 6, 5, 4, 3, 2, 1]
        ctrl_data = 10 * [1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0]
        expected_result = 10 * (0, 0, 0, 0, 4, 5, 6, 7,
                                8, 9, 9, 9, 9, 9, 9, 9, 9, 9)

        src = blocks.vector_source_f(src_data)
        ctrl = blocks.vector_source_b(ctrl_data)
        op = blocks.sample_and_hold_ff()
        dst = blocks.vector_sink_f()

        self.tb.connect(src, (op, 0))
        self.tb.connect(ctrl, (op, 1))
        self.tb.connect(op, dst)
        self.tb.run()

        result = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result, places=6)


if __name__ == '__main__':
    gr_unittest.run(test_sample_and_hold)



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import os
import math

from gnuradio import gr, gr_unittest, blocks
import pmt


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [amp * math.cos(2. * math.pi * freq * x) +
         1j * amp * math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_file_metadata(gr_unittest.TestCase):

    def setUp(self):
        os.environ['GR_CONF_CONTROLPORT_ON'] = 'False'
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        N = 1000
        outfile = "test_out.dat"

        detached = False
        samp_rate = 200000
        key = pmt.intern("samp_rate")
        val = pmt.from_double(samp_rate)
        extras = pmt.make_dict()
        extras = pmt.dict_add(extras, key, val)

        data = sig_source_c(samp_rate, 1000, 1, N)
        src = blocks.vector_source_c(data)
        fsnk = blocks.file_meta_sink(gr.sizeof_gr_complex, outfile,
                                     samp_rate, 1,
                                     blocks.GR_FILE_FLOAT, True,
                                     1000000, extras, detached)
        fsnk.set_unbuffered(True)

        self.tb.connect(src, fsnk)
        self.tb.run()
        fsnk.close()

        handle = open(outfile, "rb")
        header_str = handle.read(blocks.parse_file_metadata.HEADER_LENGTH)
        self.assertGreater(len(header_str), 0)

        header = pmt.deserialize_str(header_str)
        info = blocks.parse_header(header, False)

        extra_str = handle.read(info["extra_len"])
        self.assertEqual(len(extra_str) > 0, True)

        handle.close()

        extra = pmt.deserialize_str(extra_str)
        extra_info = blocks.parse_extra_dict(extra, info, False)

        self.assertEqual(info['rx_rate'], samp_rate)
        self.assertEqual(pmt.to_double(extra_info['samp_rate']), samp_rate)

        # Test file metadata source
        src.rewind()
        fsrc = blocks.file_meta_source(outfile, False)
        vsnk = blocks.vector_sink_c()
        tsnk = blocks.tag_debug(gr.sizeof_gr_complex, "QA")
        ssnk = blocks.vector_sink_c()
        self.tb.disconnect(src, fsnk)
        self.tb.connect(fsrc, vsnk)
        self.tb.connect(fsrc, tsnk)
        self.tb.connect(src, ssnk)
        self.tb.run()

        fsrc.close()
        # Test to make sure tags with 'samp_rate' and 'rx_rate' keys
        # were generated and received correctly.
        tags = tsnk.current_tags()
        for t in tags:
            if(pmt.eq(t.key, pmt.intern("samp_rate"))):
                self.assertEqual(pmt.to_double(t.value), samp_rate)
            elif(pmt.eq(t.key, pmt.intern("rx_rate"))):
                self.assertEqual(pmt.to_double(t.value), samp_rate)

        # Test that the data portion was extracted and received correctly.
        self.assertComplexTuplesAlmostEqual(vsnk.data(), ssnk.data(), 5)

        os.remove(outfile)

    def test_002(self):
        N = 1000
        outfile = "test_out.dat"
        outfile_hdr = "test_out.dat.hdr"

        detached = True
        samp_rate = 200000
        key = pmt.intern("samp_rate")
        val = pmt.from_double(samp_rate)
        extras = pmt.make_dict()
        extras = pmt.dict_add(extras, key, val)

        data = sig_source_c(samp_rate, 1000, 1, N)
        src = blocks.vector_source_c(data)
        fsnk = blocks.file_meta_sink(gr.sizeof_gr_complex, outfile,
                                     samp_rate, 1,
                                     blocks.GR_FILE_FLOAT, True,
                                     1000000, extras, detached)
        fsnk.set_unbuffered(True)

        self.tb.connect(src, fsnk)
        self.tb.run()
        fsnk.close()

        # Open detached header for reading
        handle = open(outfile_hdr, "rb")
        header_str = handle.read(blocks.parse_file_metadata.HEADER_LENGTH)
        self.assertGreater(len(header_str), 0)

        header = pmt.deserialize_str(header_str)
        info = blocks.parse_header(header, False)

        extra_str = handle.read(info["extra_len"])

        self.assertEqual(len(extra_str) > 0, True)

        handle.close()

        extra = pmt.deserialize_str(extra_str)
        extra_info = blocks.parse_extra_dict(extra, info, False)

        self.assertEqual(info['rx_rate'], samp_rate)
        self.assertEqual(pmt.to_double(extra_info['samp_rate']), samp_rate)

        # Test file metadata source
        src.rewind()
        fsrc = blocks.file_meta_source(outfile, False, detached, outfile_hdr)
        vsnk = blocks.vector_sink_c()
        tsnk = blocks.tag_debug(gr.sizeof_gr_complex, "QA")
        ssnk = blocks.vector_sink_c()
        self.tb.disconnect(src, fsnk)
        self.tb.connect(fsrc, vsnk)
        self.tb.connect(fsrc, tsnk)
        self.tb.connect(src, ssnk)
        self.tb.run()

        fsrc.close()
        # Test to make sure tags with 'samp_rate' and 'rx_rate' keys
        # were generated and received correctly.
        tags = tsnk.current_tags()
        for t in tags:
            if(pmt.eq(t.key, pmt.intern("samp_rate"))):
                self.assertEqual(pmt.to_double(t.value), samp_rate)
            elif(pmt.eq(t.key, pmt.intern("rx_rate"))):
                self.assertEqual(pmt.to_double(t.value), samp_rate)

        # Test that the data portion was extracted and received correctly.
        self.assertComplexTuplesAlmostEqual(vsnk.data(), ssnk.data(), 5)

        os.remove(outfile)
        os.remove(outfile_hdr)


if __name__ == '__main__':
    gr_unittest.run(test_file_metadata)



""""""

import functools
import grc.converter.cheetah_converter as parser


def test_basic():
    c = parser.Converter(names={'abc'})
    for convert in (c.convert_simple, c.convert_hard, c.to_python):
        assert 'abc' == convert('$abc')
        assert 'abc' == convert('$abc()')
        assert 'abc' == convert('$(abc)')
        assert 'abc' == convert('$(abc())')
        assert 'abc' == convert('${abc}')
        assert 'abc' == convert('${abc()}')

    assert c.stats['simple'] == 2 * 6
    assert c.stats['hard'] == 1 * 6


def test_simple():
    convert = parser.Converter(names={'abc': {'def'}})
    assert 'abc' == convert.convert_simple('$abc')
    assert 'abc.def' == convert.convert_simple('$abc.def')
    assert 'abc.def' == convert.convert_simple('$(abc.def)')
    assert 'abc.def' == convert.convert_simple('${abc.def}')
    try:
        convert.convert_simple('$abc.not_a_sub_key')
    except NameError:
        assert True
    else:
        assert False


def test_conditional():
    convert = parser.Converter(names={'abc'})
    assert '(asb_asd_ if abc > 0 else __not__)' == convert.convert_inline_conditional(
        '#if $abc > 0 then asb_$asd_ else __not__')


def test_simple_format_string():
    convert = functools.partial(parser.Converter(
        names={'abc'}).convert_simple, spec=parser.FormatString)
    assert '{abc}' == convert('$abc')
    assert '{abc:eval}' == convert('$abc()')
    assert '{abc}' == convert('$(abc)')
    assert '{abc:eval}' == convert('$(abc())')
    assert '{abc}' == convert('${abc}')
    assert '{abc:eval}' == convert('${abc()}')


def test_hard_format_string():
    names = {'abc': {'ff'}, 'param1': {}, 'param2': {}}
    convert = functools.partial(parser.Converter(
        names).convert_hard, spec=parser.FormatString)
    assert 'make_a_cool_block_{abc.ff}({param1}, {param2})' == \
           convert('make_a_cool_block_${abc.ff}($param1, $param2)')


converter = parser.Converter(names={'abc'})
c2p = converter.to_python


def test_opts():
    assert 'abc abc abc' == c2p('$abc $(abc) ${abc}')
    assert 'abc abc.abc abc' == c2p('$abc $abc.abc ${abc}')
    assert 'abc abc[''].abc abc' == c2p('$abc $abc[''].abc() ${abc}')


def test_nested():
    assert 'abc(abc) abc + abc abc[abc]' == c2p(
        '$abc($abc) $(abc + $abc) ${abc[$abc]}')
    assert '(abc_abc_)' == c2p('(abc_$(abc)_)')


def test_nested2():
    class Other(parser.Python):
        nested_start = '{'
        nested_end = '}'
    assert 'abc({abc})' == converter.convert('$abc($abc)', spec=Other)


def test_nested3():
    class Other(parser.Python):
        start = '{'
        end = '}'
    assert '{abc(abc)}' == converter.convert('$abc($abc)', spec=Other)


def test_with_string():
    assert 'abc "$(abc)" abc' == c2p('$abc "$(abc)" ${abc}')
    assert 'abc \'$(abc)\' abc' == c2p('$abc \'$(abc)\' ${abc}')
    assert 'abc "\'\'$(abc)" abc' == c2p('$abc "\'\'$(abc)" ${abc}')


def test_if():
    result = converter.to_mako("""
        #if $abc > 0
            test
        #else if $abc < 0
            test
        #else
            bla
        #end if
    """)

    expected = """
        % if abc > 0:
            test
        % elif abc < 0:
            test
        % else:
            bla
        % endif
    """
    assert result == expected


def test_hash_end():
    result = converter.to_mako('$abc#slurp')
    assert result == '${abc}\\'


def test_slurp_if():
    result = converter.to_mako("""
        $abc#slurp
        #if $abc
    """)

    expected = """
        ${abc}
        % if abc:
    """
    assert result == expected



#!/usr/bin/env python
#
# Copyright 2011, 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, qtgui


class test_qtgui(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    # Tests to make sure we can instantiate the sink.
    # We use '5' in place of fft.window.WIN_BLACKMAN_hARRIS so we
    # don't have to worry about importing fft just for this one
    # constant.
    def test01(self):
        self.qtsnk = qtgui.sink_c(1024, 5,
                                  0.0, 1.0, "Test",
                                  True, True, True, True,
                                  None)

    def test02(self):
        self.qtsnk = qtgui.sink_f(1024, 5,
                                  0, 1, "Test",
                                  True, True, True, True,
                                  None)

    def test03(self):
        self.qtsnk = qtgui.time_sink_c(1024, 1, "Test", 1, None)

    def test04(self):
        self.qtsnk = qtgui.time_sink_f(1024, 1, "Test", 1, None)

    def test05(self):
        self.qtsnk = qtgui.freq_sink_c(1024, 5,
                                       0, 1, "Test", 1, None)

    def test06(self):
        self.qtsnk = qtgui.freq_sink_f(1024, 5,
                                       0, 1, "Test", 1, None)

    def test07(self):
        self.qtsnk = qtgui.waterfall_sink_c(1024, 5,
                                            0, 1, "Test", 1, None)

    def test08(self):
        self.qtsnk = qtgui.waterfall_sink_f(1024, 5,
                                            0, 1, "Test", 1, None)

    def test09(self):
        self.qtsnk = qtgui.const_sink_c(1024, "Test", 1, None)

    def test10(self):
        self.qtsnk = qtgui.time_raster_sink_b(1024, 100, 100.5,
                                              [], [], "Test", 1, None)

    def test11(self):
        self.qtsnk = qtgui.time_raster_sink_f(1024, 100, 100.5,
                                              [], [], "Test", 1, None)

    def test12(self):
        self.qtsnk = qtgui.histogram_sink_f(1024, 100, -1, 1, "Test", 1, None)

    def test13(self):
        self.qtsnk = qtgui.eye_sink_f(1024, 1, 1, None)

    def test14(self):
        self.qtsnk = qtgui.eye_sink_c(1024, 1, 1, None)


if __name__ == '__main__':
    gr_unittest.run(test_qtgui)



#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Quantizer
# Generated: Thu Aug  1 11:09:51 2013
##################################################

from gnuradio import blocks
from gnuradio import gr
from gnuradio.filter import firdes


class quantizer(gr.hier_block2):

    def __init__(self, bits=16):
        gr.hier_block2.__init__(
            self, "Quantizer",
            gr.io_signature(1, 1, gr.sizeof_float * 1),
            gr.io_signature(1, 1, gr.sizeof_float * 1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.bits = bits

        ##################################################
        # Blocks
        ##################################################
        self.blocks_short_to_float_0 = blocks.short_to_float(1, 1)
        self.blocks_multiply_const_vxx_0_0 = blocks.multiply_const_vff(
            (1.0 / pow(2.0, bits - 1.0), ))
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vff(
            (pow(2, bits - 1.0), ))
        self.blocks_float_to_short_0 = blocks.float_to_short(1, 1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_multiply_const_vxx_0_0, 0), (self, 0))
        self.connect((self.blocks_short_to_float_0, 0),
                     (self.blocks_multiply_const_vxx_0_0, 0))
        self.connect((self.blocks_float_to_short_0, 0),
                     (self.blocks_short_to_float_0, 0))
        self.connect((self.blocks_multiply_const_vxx_0, 0),
                     (self.blocks_float_to_short_0, 0))
        self.connect((self, 0), (self.blocks_multiply_const_vxx_0, 0))

    def get_bits(self):
        return self.bits

    def set_bits(self, bits):
        self.bits = bits
        self.blocks_multiply_const_vxx_0_0.set_k(
            (1.0 / pow(2.0, self.bits - 1.0), ))
        self.blocks_multiply_const_vxx_0.set_k((pow(2, self.bits - 1.0), ))



"""
GNU Radio is a free & open-source software development toolkit that provides signal processing blocks to implement software radios. It can be used with readily-available low-cost external RF hardware to create software-defined radios, or without hardware in a simulation-like environment. It is widely used in hobbyist, academic and commercial environments to support both wireless communications research and real-world radio systems.

GNU Radio applications are primarily written using the Python programming language, while the supplied performance-critical signal-processing path is implemented in C++ using processor floating-point extensions, where available. Thus, the developer is able to implement real-time, high-throughput radio systems in a simple-to-use, rapid-application-development environment.

While not primarily a simulation tool, GNU Radio does support development of signal processing algorithms using pre-recorded or generated data, avoiding the need for actual RF hardware.

GNU Radio is licensed under the GNU General Public License (GPL) version 3. All of the code is copyright of the Free Software Foundation.
"""

# This file makes gnuradio a package
# The docstring will be associated with the top level of the package.

import os
from pkgutil import extend_path

# Allow OOT modules to be installed elsewhere on the PYTHONPATH
# For example, gnuradio might be installed to /usr/lib/python3.8/site-packages/gnuradio
#  But an OOT might be installed to /some/other/directory/gnuradio/myoot
#  As long as /some/other/directory/ is in PYTHONPATH, this init file should find it
#  and `from gnuradio import myoot` will work as expected
__path__ = extend_path(__path__, __name__)

# python3.8 and up need to have the dll search path set
# https://docs.python.org/3/whatsnew/3.8.html#bpo-36085-whatsnew
if os.name == 'nt' and hasattr(os, 'add_dll_directory'):
    root_dir = __file__
    for i in range(5):  # limit search depth
        root_dir = os.path.dirname(root_dir)
        bin_dir = os.path.join(root_dir, 'bin')
        if os.path.exists(bin_dir):
            try:
                os.add_dll_directory(bin_dir)
            except Exception as ex:
                print('add_dll_directory(%s): %s' % (bin_dir, ex))
            break

# Check if the gnuradio package is installed or whether we're attempting to import it from
# the build directory.
path_ending = os.path.join(
    'gnuradio-runtime', 'python', 'gnuradio', '__init__.py')
path = os.path.abspath(__file__)
if path.endswith('.pyc'):
    path = path[:-1]

if path.endswith(path_ending):
    # We importing it from build directory.
    build_path = os.path.join(path[:-len(path_ending)])

    # Place these directories on __path__ so that their contents are
    # part of the gnuradio package.
    __path__.append(os.path.join(build_path, 'gr-utils', 'src', 'python'))
    __path__.append(os.path.join(build_path, 'gr-blocks', 'python'))
    __path__.append(os.path.join(build_path, 'gr-digital', 'python'))
    __path__.append(os.path.join(build_path, 'gr-filter', 'python'))
    __path__.append(os.path.join(build_path, 'gr-fft', 'python'))
    __path__.append(os.path.join(build_path, 'gr-analog', 'python'))
    __path__.append(os.path.join(build_path, 'gr-dtv', 'python'))
    __path__.append(os.path.join(build_path, 'gr-trellis', 'python'))
    __path__.append(os.path.join(build_path, 'gr-wavelet', 'python'))
    __path__.append(os.path.join(build_path, 'gr-audio', 'python'))
    __path__.append(os.path.join(build_path, 'gr-qtgui', 'python'))
    __path__.append(os.path.join(build_path, 'gr-video-sdl', 'python'))
    __path__.append(os.path.join(build_path, 'gr-vocoder', 'python'))
    __path__.append(os.path.join(build_path, 'gr-channels', 'python'))
    __path__.append(os.path.join(build_path, 'gr-fec', 'python'))
    __path__.append(os.path.join(build_path, 'gr-utils'))
    __path__.append(os.path.join(build_path, 'gr-iio', 'python'))
    __path__.append(os.path.join(build_path, 'gr-uhd', 'python'))
    __path__.append(os.path.join(build_path, 'gr-pdu', 'python'))
    __path__.append(os.path.join(build_path, 'gr-network', 'python'))
    __path__.append(os.path.join(build_path, 'gr-zeromq', 'python'))
    __path__.append(os.path.join(build_path, 'gr-soapy', 'python'))



#!/usr/bin/env python
#
# Copyright 2008,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt
import math


def make_tag(key, value, offset, srcid=None):
    tag = gr.tag_t()
    tag.key = pmt.string_to_symbol(key)
    tag.value = pmt.to_pmt(value)
    tag.offset = offset
    if srcid is not None:
        tag.srcid = pmt.to_pmt(srcid)
    return tag


def compare_tags(a, b):
    return a.offset == b.offset and pmt.equal(a.key, b.key) and \
        pmt.equal(a.value, b.value) and pmt.equal(a.srcid, b.srcid)


class test_vector_sink_source(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        # Test that sink has data set in source for the simplest case
        src_data = [float(x) for x in range(16)]
        expected_result = src_data

        src = blocks.vector_source_f(src_data)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_002(self):
        # Test vectors (the gnuradio vector I/O type)
        src_data = [float(x) for x in range(16)]
        expected_result = src_data

        src = blocks.vector_source_f(src_data, False, 2)
        dst = blocks.vector_sink_f(2)

        self.tb.connect(src, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_003(self):
        # Test that we can only make vectors (the I/O type) if the input
        # vector has sufficient size
        src_data = [float(x) for x in range(16)]
        self.assertRaises(
            ValueError, lambda: blocks.vector_source_f(
                src_data, False, 3))

    def test_004(self):
        # Test sending and receiving tagged streams
        src_data = [float(x) for x in range(16)]
        expected_result = src_data
        src_tags = [make_tag('key', 'val', 0, 'src')]
        expected_tags = src_tags[:]

        src = blocks.vector_source_f(src_data, repeat=False, tags=src_tags)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(expected_result, result_data)
        self.assertEqual(len(result_tags), 1)
        self.assertTrue(compare_tags(expected_tags[0], result_tags[0]))

    def test_005(self):
        # Test that repeat works (with tagged streams)
        length = 16
        src_data = [float(x) for x in range(length)]
        expected_result = src_data + src_data
        src_tags = [make_tag('key', 'val', 0, 'src')]
        expected_tags = [make_tag('key', 'val', 0, 'src'),
                         make_tag('key', 'val', length, 'src')]

        src = blocks.vector_source_f(src_data, repeat=True, tags=src_tags)
        head = blocks.head(gr.sizeof_float, 2 * length)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, head, dst)
        self.tb.run()
        result_data = dst.data()
        result_tags = dst.tags()
        self.assertEqual(expected_result, result_data)
        self.assertEqual(len(result_tags), 2)
        self.assertTrue(compare_tags(expected_tags[0], result_tags[0]))
        self.assertTrue(compare_tags(expected_tags[1], result_tags[1]))

    def test_006(self):
        # Test set_data
        src_data = [float(x) for x in range(16)]
        expected_result = src_data

        src = blocks.vector_source_f((3, 1, 4))
        dst = blocks.vector_sink_f()
        src.set_data(src_data)

        self.tb.connect(src, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)

    def test_007(self):
        # Test set_repeat
        src_data = [float(x) for x in range(16)]
        expected_result = src_data

        src = blocks.vector_source_f(src_data, True)
        dst = blocks.vector_sink_f()
        src.set_repeat(False)

        self.tb.connect(src, dst)
        # will timeout if set_repeat does not work
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_vector_sink_source)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import os
import struct


class test_tag_file_sink(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        trg_data = [-1, -1, 1, 1, -1, -1, 1, 1, -1, -1]
        src = blocks.vector_source_i(src_data)
        trg = blocks.vector_source_s(trg_data)
        op = blocks.burst_tagger(gr.sizeof_int)
        snk = blocks.tagged_file_sink(gr.sizeof_int, 1)
        self.tb.connect(src, (op, 0))
        self.tb.connect(trg, (op, 1))
        self.tb.connect(op, snk)
        self.tb.run()

        # Tagged file sink gets 2 burst tags at index 2 and index 5.
        # Creates two new files, each with two integers in them from
        # src_data at these indexes (3,4) and (7,8).
        file0 = "file{0}_0_2.00000000.dat".format(snk.unique_id())
        file1 = "file{0}_1_6.00000000.dat".format(snk.unique_id())

        # Open the files and read in the data, then remove the files
        # to clean up the directory.
        outfile0 = open(file0, 'rb')
        outfile1 = open(file1, 'rb')
        data0 = outfile0.read(8)
        data1 = outfile1.read(8)
        outfile0.close()
        outfile1.close()
        os.remove(file0)
        os.remove(file1)

        # Convert the 8 bytes from the files into a tuple of 2 ints.
        idata0 = struct.unpack('ii', data0)
        idata1 = struct.unpack('ii', data1)

        self.assertEqual(idata0, (3, 4))
        self.assertEqual(idata1, (7, 8))


if __name__ == '__main__':
    gr_unittest.run(test_tag_file_sink)



#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

###############################################################################
# Imports
###############################################################################
from gnuradio import zeromq
from gnuradio import gr
from gnuradio import blocks
from gnuradio import analog
from gnuradio import eng_notation
from gnuradio.eng_arg import eng_float, intx
from argparse import ArgumentParser
import numpy
import sys
from threading import Thread
import time


###############################################################################
# GNU Radio top_block
###############################################################################
class top_block(gr.top_block):
    def __init__(self, options):
        gr.top_block.__init__(self)

        self.options = options

        # socket addresses
        rpc_adr = "tcp://*:6666"
        probe_adr = "tcp://*:5556"
        sink_adr = "tcp://*:5555"

        # the strange sampling rate gives a nice movement in the plot :P
        self.samp_rate = samp_rate = 48200

        # blocks
        self.gr_sig_source = analog.sig_source_f(
            samp_rate, analog.GR_SIN_WAVE, 1000, 1, 0)
        self.throttle = blocks.throttle(gr.sizeof_float, samp_rate)
        self.mult = blocks.multiply_const_ff(1)
        #self.zmq_sink = zeromq.rep_sink(gr.sizeof_float, 1, sink_adr)
        self.zmq_sink = zeromq.pub_sink(gr.sizeof_float, 1, sink_adr)
        #self.zmq_sink = zeromq.push_sink(gr.sizeof_float, 1, sink_adr)
        #self.zmq_probe = zeromq.push_sink(gr.sizeof_float, 1, probe_adr)
        self.zmq_probe = zeromq.pub_sink(gr.sizeof_float, 1, probe_adr)
        #self.null_sink = blocks.null_sink(gr.sizeof_float)

        # connects
        self.connect(self.gr_sig_source, self.mult,
                     self.throttle, self.zmq_sink)
        self.connect(self.throttle, self.zmq_probe)

        # ZeroMQ
        self.rpc_manager = zeromq.rpc_manager()
        self.rpc_manager.set_reply_socket(rpc_adr)
        self.rpc_manager.add_interface("start_fg", self.start_fg)
        self.rpc_manager.add_interface("stop_fg", self.stop_fg)
        self.rpc_manager.add_interface("set_waveform", self.set_waveform)
        self.rpc_manager.add_interface("set_k", self.mult.set_k)
        self.rpc_manager.add_interface(
            "get_sample_rate", self.throttle.sample_rate)
        self.rpc_manager.start_watcher()

    def start_fg(self):
        print("Start Flowgraph")
        try:
            self.start()
        except RuntimeError:
            print("Can't start, flowgraph already running!")

    def stop_fg(self):
        print("Stop Flowgraph")
        self.stop()
        self.wait()

    def set_waveform(self, waveform_str):
        waveform = {'Constant': analog.GR_CONST_WAVE,
                    'Sine': analog.GR_SIN_WAVE,
                    'Cosine': analog.GR_COS_WAVE,
                    'Square': analog.GR_SQR_WAVE,
                    'Triangle': analog.GR_TRI_WAVE,
                    'Saw Tooth': analog.GR_SAW_WAVE}[waveform_str]
        self.gr_sig_source.set_waveform(waveform)

###############################################################################
# Options Parser
###############################################################################


def parse_args():
    """Argument parser."""
    parser = ArgumentParser()
    args = parser.parse_args()
    return args


###############################################################################
# Main
###############################################################################
if __name__ == "__main__":
    args = parse_args()
    tb = top_block(args)
    try:
        # keep the program running when flowgraph is stopped
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        pass
    print("Shutting down flowgraph.")
    tb.rpc_manager.stop_watcher()
    tb.stop()
    tb.wait()
    tb = None
